#!/usr/bin/perl 

################################################################################
# 
# PROGRAM:  Admin.pl - from closedShop - Open Shopping Cart 
#
# RCSID:  $Id$
#
# DESCRIPTION: Admininstration for shopping cart
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:  
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-05-03   cbf    Created
# 2001-06-07   cbf    Added attributes to products
# 2001-06-09   cbf    Added a field for product id specified by merchant
# 2001-06-11   cbf    Removed image file if file was not uploaded
# 2001-06-20   cbf    Added ability to change program variables
# 2001-06-22   cbf    Added ability to change filenames
# 2001-06-24   cbf    When displaying order, filename wasn't correct on a link
# 2001-07-24   cbf    Mailprogram can be customized
# 2001-08-05   cbf    Attributes weren't showing in admin screen
#                     All input is validated
# 2001-08-12   cbf    Added instructions about "processing orders"
# 2001-09-08   cbf    Added SecureImageURL variable use
# 2001-09-13   cbf    Made Edit/Delete list be sorted by category, put edit
#                      button next to each item, to reduce scroll time
# 2001-09-17   cbf    Let admin remove customers from cust. list
# 2001-09-18   cbf    Let admin email all customers a la mail program
# 2001-09-27   dsb    Change variables takes account of Merchant/Admin uname
# 2001-10-18   cbf    Added database location
# 2002-01-20   cbf    Each category now has variables to be edited
# 2002-01-29   cbf    Rudimentary Download Order History Log
# 2002-02-11   cbf    Let categories be public or private
# 2002-05-12   cbf    Added minimum order amount, added continue shopping vars,
#                       final return vars
# 2002-05-12   cbf    Removed extra space from customer section
################################################################################

use CGI::Carp(fatalsToBrowser);
use closedShop;
use strict;
use CGI;
use DBI;
use Fcntl ':flock';

closedShop::Initialize();

###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query = new CGI;
my $program_name = "$closedShop::FileNames{admin}";
	
my %actions = (AddProduct => "Add Product",
	           EditProduct => "Edit Product",
			   DeleteProduct => "Delete Product",
			   ViewOrders => "View / Process Orders",
			   ViewCustomers => "View customers",
			   AdminScreen => "Administration Panel",
			   ChangeVariables => "Edit Program Variables",
			   EditCategories => "Edit Category Variables",
			   DownloadOrder => "Download Order History Log"
				);
################################################################################


Main();
closedShop::Cleanup();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main
{
	my $action = $query->param('action'); 
	$action = "cs" if (!defined($action));
	if ($action eq "Login") {
	  Login();
	}
	elsif ($actions{$action} eq "") { 
	  LoginScreen("Please login to use the fuctionality");
	}
	else {
      my ($session,$uid) = closedShop::GetSessionCookie();
	  if (!closedShop::Authenticate($session,"admin")) {
	    return LoginScreen("You are not logged in correctly");
	  }
	  eval ($action);
	}
}

###############################################################################
# FUNCTION:  DownloadOrder
# DESCRIPTION: download order log
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DownloadOrder
{
	print "Content-type: text/html \n\n";

	my $SQL	= qq| select id,DATE_FORMAT(date_bought,"%W, %M %d %Y"),
	              total,b_name,b_address,b_city,b_state,
                  b_zip,b_phone,s_name,s_address,s_city,s_state,
                  s_zip,s_phone,cc_name,cc,cc_num,cc_month,cc_year
				  from $closedShop::sql_tables{"order"} 
		          where complete="Y" order by date_bought  |;
	my $sth = closedShop::DatabaseQuery($SQL);
	
	foreach (qw/ID Date_Bought Total Billing_Name Billing_Address Billing_City
				Billing_State Billing_Zip Billing_Phone Shipping_Name
				Shipping_Address Shipping_City Shipping_State
				Shipping_Zip Shipping_Phone CC_Name Credit_Card
				CC_Number CC_Month CC_Year/) {
		print qq|"$_",|;		
	}
	print qq|"Count"<BR>\n|;
	
	my $k = 1;
	while (my (@ary) = $sth->fetchrow_array()) {
	  foreach (@ary) {
		print qq|"$_",|;
	  }
	  print qq|"$k"<BR>\n|;
	  $k++;
	}
}

###############################################################################
# FUNCTION:  EmailCustomers
# DESCRIPTION: emails all customers
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub EmailCustomers
{
	my ($subject,$body) = @_;
	my $k = 0;
	my $pid = fork();
	
	if ($pid) {
	  return;
	}
	else {
	  close (STDOUT);
	  my $SQL	= qq| select name,email from $closedShop::sql_tables{"user"} 
	  where email != "" order by name|;
      my $sth = closedShop::DatabaseQuery($SQL);
	  
	  while (my ($name,$email) = $sth->fetchrow_array()) {
		closedShop::SendEmail($email,$subject,$body);
		$k++;
	  }
	}	
}

###############################################################################
# FUNCTION:  EditCategories
# DESCRIPTION: add a product to database
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub EditCategories
{
	if ($query->param("selected") == 1) {
	  open(FILE,">$closedShop::categoryfile") ||
	    return AdminScreen("Can't open category file - $closedShop::categoryfile");
	  flock(FILE,LOCK_EX);
	  foreach (sort keys %closedShop::categories) {
	    my $intro = $query->param("intro_$_"); 
		$intro =~ s/\n//g; $intro =~ s/~//g;
		my $style = $query->param("style_$_");
		$style =~ s/~//g;
		my $numimages = $query->param("numimages_$_");
		my $status = $query->param("status_$_");
	    my $data = join("~",($_,$intro,$style,$numimages,$status));
		print FILE qq|$data\n|;
	  }
	  flock(FILE,LOCK_UN);
	  close(FILE);	  
	  return AdminScreen("Categories Edited Successfully");
	}
	else {
	  closedShop::Header();
	  print qq|
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="EditCategories">
	  <input type="hidden" name="selected" value="1">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	  $closedShop::font <font size="3"><font color="blue">Edit Categories</font></th></tr>
      |; 	 
	 
	  
	  foreach (sort keys %closedShop::categories) {
	    my ($intro,$style,$numimages,$status) = closedShop::GetCategoryInfo($_);
		my $checktext = ""; my $checkthumb = "";
		my $checkprivate = ""; my $checkpublic = "";
		if ($style eq "thumbnail") { $checkthumb = "checked"; }
		else { $checktext = "checked"; }
		
		if ($status eq "PRIVATE") { $checkprivate = "checked"; }
		else { $checkpublic = "checked"; }
		
		print qq|
          <tr bgcolor="$closedShop::mc"><td>
		  <center>$closedShop::font $closedShop::categories{$_}</td>
	      <td>$closedShop::font <b>Intro Text</b><br>
		  <textarea rows="5" cols="35" name="intro_$_">$intro</textarea><br>
		  <br>
		  <b>Category Display Type</b><br>
		  <input type="radio" name="style_$_" $checktext value="text"> Text Display
		  &nbsp;&nbsp;&nbsp;
		  <input type="radio" name="style_$_" $checkthumb value="thumbnail"> Thumbnail Display<br>
		  Number of thumbnail
		  images to display: <select name="numimages_$_">
		  |;
		  if ($numimages) {
		    print qq|<option value="$numimages">$numimages Items</option>|;
		  }
		  print qq|
		  <option value="16">16 Items</option>
		  <option value="8">8 Items</option>
		  <option value="4">4 Items</option>
		  <option value="2">2 Items</option>
		  <option value="1">1 Item</option>
		  </select><br>
		  
		  <br><strong>Status of category</strong><br>
		  <input type="radio" name="status_$_" $checkpublic value="PUBLIC"> Public (Visible) 
		  &nbsp;&nbsp;&nbsp;
		  <input type="radio" name="status_$_" $checkprivate value="PRIVATE"> 
		  Private (Not-Visible)<br>
		  </td></tr>
		|;
	  }
	  
	  print qq|
	  <tr bgcolor="$closedShop::mc">
	  <td colspan="2"><div align="center"><input type="submit" value="Edit Categories"></div></td>
	  </tr></form></table>
	  <div align="center"><form action="$program_name" method="POST">
	    <input type="hidden" name="action" value="AdminScreen">
	    <input type="submit" value="Return to Control Panel"></form></div></center>|;
	  closedShop::Footer();
	
	}

}

###############################################################################
# FUNCTION:  ViewCustomers
# DESCRIPTION: add a product to database
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ViewCustomers
{
	closedShop::Header();

	if ($query->param("selected") eq "RemoveCustomer") {
	  my $id = $query->param("id");
	  my $SQL2	= qq| delete from $closedShop::sql_tables{"user"} 
	  where id = "$id" |;
      my $sth2 = closedShop::DatabaseQuery($SQL2);
	  print qq|<div align="center">$closedShop::font
	  <font color="red">You have removed the customer</font></div><br>|;
	  goto VIEWCUSTSTART;	  
	}
	elsif ($query->param("selected") eq "EmailCustomer") {
	  my $subject = $query->param("subject");
	  my $body = $query->param("body");
	  EmailCustomers($subject,$body);
	   print qq|<div align="center">$closedShop::font
	  <font color="red">You have successfully emailed the list</font></div><br>|;
	  goto VIEWCUSTSTART;	  
	  
	}
	elsif ($query->param("selected") eq "EmailCustomerDisplay") {
	  print qq|
	  <form action="$program_name" method="POST">
	   <input type="hidden" name="action" value="ViewCustomers">
	   <input type="hidden" name="selected" value="EmailCustomer">
	   <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	  $closedShop::font <font size="3"><font color="blue">Email Customers</font></th></tr>
 	  <tr bgcolor="$closedShop::mc">
		<td>$closedShop::font Subject</td>
		<td>$closedShop::font <input type="text" name="subject" size="30"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Body</td>
	  <td>$closedShop::font <textarea name="body" cols="30" rows="5" wrap="physical"></textarea></td>
	  <tr bgcolor="$closedShop::mc">
	  <td colspan="2"><div align="center"><input type="submit" value="Send Emails"></div></td>
	  </tr></form></table>
	  <div align="center"><form action="$program_name" method="POST">
	    <input type="hidden" name="action" value="AdminScreen">
	    <input type="submit" value="Return to Control Panel"></form></div></center>|;
	  closedShop::Footer();
	  	  
	}
	else {
	  VIEWCUSTSTART:
	  
	  my $SQL	= qq| select name,email,address,city,state,zip,phone,id
	  from $closedShop::sql_tables{"user"} where email != "" order by name|;
      my $sth = closedShop::DatabaseQuery($SQL);
	  
	  print qq|
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="5">
	  $closedShop::font <font size="3"><font color="blue">View Customers</font></th></tr>
 	  <tr bgcolor="$closedShop::mc">
		<td>$closedShop::font<b>Name</b></td><td>$closedShop::font<b>Email</b></td>
	  <td>$closedShop::font<b>Address/<br>Phone</b></td>
	  <td>$closedShop::font<b>Total Spent</b></td>
	  <td>$closedShop::font<b>Remove</b></td>
	  </tr>|;
	  my $k = 0;
	  while (my ($name,$email,$address,$city,
	  $state,$zip,$phone,$id) = $sth->fetchrow_array()) {
	   print qq|
	   <tr bgcolor="$closedShop::mc">
		 
		<td><form action="$program_name" method="POST">
	   <input type="hidden" name="action" value="ViewCustomers">
	   <input type="hidden" name="selected" value="RemoveCustomer">
	   <input type="hidden" name="id" value="$id">$closedShop::font $name</td>
		
		<td>$closedShop::font<a href="mailto:$email">$email</a></td>
	    <td>$closedShop::font $address<br>$city, $state $zip<br>$phone</td>
	    |;
	 
	  my $SQL2	= qq| select sum(total) from $closedShop::sql_tables{"order"} where
	  date_processed is NOT NULL and user_id = "$id"|;
	  my $sth2 = closedShop::DatabaseQuery($SQL2);
	  my ($sum) = $sth2->fetchrow_array();
	  if (!$sum) { $sum = "0.00"; }
	  
	  print qq|<td>$closedShop::font \$$sum</td>
	  <td>$closedShop::font <input type="submit" value="Remove"></td></form>
	  </tr>|;
	  $k++;
	 }
	 
	 print qq|<tr bgcolor="$closedShop::mc">
		<td colspan="5">$closedShop::font <div align="center"><b>Total Customers: $k</b></div></td></tr>
	    </table></form>
		<br>
		
		<div align="center"><form action="$program_name" method="POST">
	   <input type="hidden" name="action" value="ViewCustomers">
	   <input type="hidden" name="selected" value="EmailCustomerDisplay">
	   <input type="submit" value="Email All Customers"></form></div></center>
	   
	   </div>
		

		
		<div align="center"><form action="$program_name" method="POST">
	    <input type="hidden" name="action" value="AdminScreen">
	    <input type="submit" value="Return to Control Panel"></form></div></center>|;
	  closedShop::Footer();
	}
}

###############################################################################
# FUNCTION:  AddProduct
# DESCRIPTION: add a product to database
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub AddProduct
{
	if ($query->param("selected") == 1) {
	  my $name = $query->param("name"); 
	  $name =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $category = $query->param("category"); 
	  $category =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $price = $query->param("price"); 
	  $price =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $code = $query->param("code"); 
	  $code =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $shipping = $query->param("shipping"); 
	  $shipping =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $desc = $query->param("desc"); 
	  $desc =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $image = $query->param("image"); 
	  
	  my @attr1 = (); my @attr2 = (); my @attr3 = ();
	  my $k = 1;
	  while ($query->param("attribute1_$k")) {
	    my $tmp = $query->param("attribute1_$k"); $tmp =~ s/~/*/g;
	    push @attr1,$tmp;
		$k++;
	  }
	  $k = 1;
	  while ($query->param("attribute2_$k")) {
	    my $tmp = $query->param("attribute2_$k"); $tmp =~ s/~/*/g;
	    push @attr2,$tmp;
		$k++;
	  }
	  $k = 1;
	  while ($query->param("attribute3_$k")) {
	    my $tmp = $query->param("attribute3_$k"); $tmp =~ s/~/*/g;
	    push @attr3,$tmp;
		$k++;
	  }
	  
	  my $attr1 = join("~",@attr1); $attr1 =~ s/\|/=/g;
	  $attr1 = $query->param("attribute1_name") . "~" . $attr1;
	  my $attr2 = join("~",@attr2); $attr2 =~ s/\|/=/g;
	  $attr2 = $query->param("attribute2_name") . "~" . $attr2;
	  my $attr3 = join("~",@attr3); $attr3 =~ s/\|/=/g;
  	  $attr3 = $query->param("attribute3_name") . "~" . $attr3;
	  
	  my $attr = $attr1 . "|" . $attr2 . "|" . $attr3;
	  
	  my $SQL	= qq| insert into $closedShop::sql_tables{"item"} set name="$name",
	                  price="$price",shipping_price="$shipping",description="$desc",
					  category="$category",attributes="$attr",product_code="$code" |;
	  my $sth = closedShop::DatabaseQuery($SQL);
	  my $new_id = $sth->{'mysql_insertid'};
	  
	  if ($image) {
	    open (OUTFILE,">$closedShop::IMAGEPATH/$new_id.jpg");
	    while (<$image>) {  print OUTFILE;  }
	    close(OUTFILE);
		
		if (!(stat("$closedShop::IMAGEPATH/$new_id.jpg"))[7]) {
		  unlink ("$closedShop::IMAGEPATH/$new_id.jpg");
		}
		
	  }
	  return AdminScreen("You have successfully added $name to the item database");

	}
	else {
	  closedShop::Header();
	  print qq|
	  <form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="AddProduct">
	  <input type="hidden" name="selected" value="1">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	  $closedShop::font <font size="3"><font color="blue">Add a product</font></th></tr>
 	 
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Product Name:</td>
	  <td>$closedShop::font <input type="text" name="name" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Product Code/Id:</td>
	  <td>$closedShop::font <input type="text" name="code" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Category:</td>
	  <td>$closedShop::font <select name="category">|;
	  foreach (sort keys %closedShop::categories) {
	    print qq|<option value="$_">$closedShop::categories{"$_"}</option>\n|;
	  }
	  print qq|
	  </select></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Product Price:</td>
	  <td>$closedShop::font \$<input type="text" name="price" size="6"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Shipping Cost:</td>
	  <td>$closedShop::font \$<input type="text" name="shipping" size="6"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Product Description:</td>
	  <td>$closedShop::font <textarea name="desc" cols="35" rows="4"></textarea></td></tr>
  	  
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Image:<br>
	  (JPEG Format)</td>
	  <td>$closedShop::font <input type="file" name="image" size="20"></td></tr>
	  
	   <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Add Attributes:</td>
	  <td>$closedShop::font 
	  <table border="0" width="100%"><tr><td>$closedShop::font
	  Attribute 1:<br>
	  Attribute Name: <input type="text" name="attribute1_name" size="10"><br>
	  <br>
	  
	  1. <input type="text" name="attribute1_1" size="6"><br>
	  2. <input type="text" name="attribute1_2" size="6"><br>
	  3. <input type="text" name="attribute1_3" size="6"><br>
	  4. <input type="text" name="attribute1_4" size="6"><br>
	  5. <input type="text" name="attribute1_5" size="6"><br>
	  </td>
	  <td>$closedShop::font
	  Attribute 2:<br>
	  Attribute Name: <input type="text" name="attribute2_name" size="10"><br>
	  <br>
	  1. <input type="text" name="attribute2_1" size="6"><br>
	  2. <input type="text" name="attribute2_2" size="6"><br>
	  3. <input type="text" name="attribute2_3" size="6"><br>
	  4. <input type="text" name="attribute2_4" size="6"><br>
	  5. <input type="text" name="attribute2_5" size="6"><br>
	  </td>
	  <td>$closedShop::font
	  Attribute 3:<br>
	  Attribute Name: <input type="text" name="attribute3_name" size="10"><br>
	  <br>
	  1. <input type="text" name="attribute3_1" size="6"><br>
	  2. <input type="text" name="attribute3_2" size="6"><br>
	  3. <input type="text" name="attribute3_3" size="6"><br>
	  4. <input type="text" name="attribute3_4" size="6"><br>
	  5. <input type="text" name="attribute3_5" size="6"><br>
	  
	  </td>
	  </tr></table>
	  
	  </td></tr>
	  
	  <tr bgcolor="$closedShop::mc"><th colspan="2">
	  $closedShop::font <font size="3"><input type="submit" value="Add Product"></th></tr>
	  </form></table><br><br>$closedShop::font<center>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="AdminScreen">
	  <input type="submit" value="Return to Control Panel"></form></center>
	  |;
	  closedShop::Footer();
	}
}

###############################################################################
# FUNCTION:  EditProduct
# DESCRIPTION: edit a product
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub EditProduct
{
	if ($query->param("selected") == 3) {
	  my $name = $query->param("name"); 
	  $name =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $category = $query->param("category"); 
	  $category =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $price = $query->param("price"); 
	  $price =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $code = $query->param("code"); 
	  $code =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $shipping = $query->param("shipping"); 
	  $shipping =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $desc = $query->param("desc"); 
	  $desc =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  my $image = $query->param("image"); 
	  my $id = $query->param("id"); 
	  $id =~ tr/0-9//cd;
	  
	  my @attr1 = (); my @attr2 = (); my @attr3 = ();
	  my $k = 1;
	  while ($query->param("attribute1_$k")) {
	    my $tmp = $query->param("attribute1_$k"); $tmp =~ s/~/*/g;
	    push @attr1,$tmp;
		$k++;
	  }
	  $k = 1;
	  while ($query->param("attribute2_$k")) {
	    my $tmp = $query->param("attribute2_$k"); $tmp =~ s/~/*/g;
	    push @attr2,$tmp;
		$k++;
	  }
	  $k = 1;
	  while ($query->param("attribute3_$k")) {
	    my $tmp = $query->param("attribute3_$k"); $tmp =~ s/~/*/g;
	    push @attr3,$tmp;
		$k++;
	  }
	  
	  my $attr1 = join("~",@attr1); $attr1 =~ s/\|/=/g;
	  $attr1 = $query->param("attribute1_name") . "~" . $attr1;
	  my $attr2 = join("~",@attr2); $attr2 =~ s/\|/=/g;
	  $attr2 = $query->param("attribute2_name") . "~" . $attr2;
	  my $attr3 = join("~",@attr3); $attr3 =~ s/\|/=/g;
  	  $attr3 = $query->param("attribute3_name") . "~" . $attr3;

	  my $attr = $attr1 . "|" . $attr2 . "|" . $attr3;
	  
	  my $SQL = qq| update $closedShop::sql_tables{"item"} set name="$name",
	                price="$price",shipping_price="$shipping",description="$desc",
					category="$category",attributes="$attr",product_code="$code" 
					where id = "$id"|;
	  my $sth = closedShop::DatabaseQuery($SQL);

	  if ($image) {
	  print "Content-type: text/html \n\n";
	  
	    open (OUTFILE,">$closedShop::IMAGEPATH/$id.jpg") || ErrorMessage("Error $!");
	    while (<$image>) {  print "$_\n"; print OUTFILE;  }
	    close(OUTFILE) || ErrorMessage("Error $!");

		
		if (!(stat("$closedShop::IMAGEPATH/$id.jpg"))[7]) {
		print "<br>
		unlinking $closedShop::IMAGEPATH/$id.jpg" . ((stat("$closedShop::IMAGEPATH/$id.jpg"))[7]);
		  unlink ("$closedShop::IMAGEPATH/$id.jpg");
		}
	  }
	  return AdminScreen("You have successfully edited $name with $id");

	}
	elsif ($query->param("selected") == 2) {
	  my $id = $query->param("id"); $id =~ tr/0-9//cd;
	  if ($id == 0 || !$id) {
	    goto START;
	  }
	  my $SQL = qq| select name,price,shipping_price,description,
					category,attributes,product_code
	                from $closedShop::sql_tables{"item"} where id = "$id" |;
	  my $sth = closedShop::DatabaseQuery($SQL);
 	  my ($name,$price,$shipping_price,$description,
	  $category,$attr,$code) = $sth->fetchrow_array();
	  
	  my @attr = split(/\|/,$attr);
	  my @attr1 = split(/~/,$attr[0]);
	  my @attr2 = split(/~/,$attr[1]);
	  my @attr3 = split(/~/,$attr[2]);
	  
	  closedShop::Header();
	  print qq|
	  <form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="EditProduct">
	  <input type="hidden" name="selected" value="3">
	  <input type="hidden" name="id" value="$id">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	  $closedShop::font <font size="3"><font color="blue">Add a product</font></th></tr>
 	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Product Name:</td>
	  <td>$closedShop::font <input type="text" name="name" value="$name" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Product Code/Id:</td>
	  <td>$closedShop::font <input type="text" name="code" value="$code" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Category:</td>
	  <td>$closedShop::font <select name="category">
	  <option value="$category">$closedShop::categories{$category}</option>
	  <option value="$category">----------</option>|;
	  foreach (sort keys %closedShop::categories) {
	    print qq|<option value="$_">$closedShop::categories{"$_"}</option>\n|;
	  }
	  print qq|
	  </select></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Product Price:</td>
	  <td>$closedShop::font <input type="text" value="$price" name="price" size="6"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Shipping Cost:</td>
	  <td>$closedShop::font <input type="text" value="$shipping_price" name="shipping" size="6"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Product Description:</td>
	  <td>$closedShop::font <textarea name="desc" cols="35" rows="4">$description</textarea></td></tr>
  	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Image:</td>
	  <td>$closedShop::font |;
	  
	  if (-e "$closedShop::IMAGEPATH/$id.jpg") {
	    if ($closedShop::FileNames{"item"} =~ /https/) {
		  print qq|<img src="$closedShop::SecureImageURL/$id.jpg"><br>\n|;
		}
	    else {
	      print qq|<img src="$closedShop::IMAGEURL/$id.jpg"><br>\n|;
		}
	  }
	  else {
	    print qq|$closedShop::font<b>No image available</b><br>\n|;
	  }
	  
	  print qq|
	  <input type="file" name="image" size="20"><br>
	  (JPEG Format)</td></tr>
	   <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Add Attributes:</td>
	  <td>$closedShop::font 
	  <table border="0"  width="100%"><tr><td>$closedShop::font
	  Attribute 1:<br>
	  Attribute Name: <input type="text" name="attribute1_name"  value="$attr1[0]"  size="10"><br>
	  <br>
	  1. <input type="text" name="attribute1_1" value="$attr1[1]" size="6"><br>
	  2. <input type="text" name="attribute1_2"  value="$attr1[2]" size="6"><br>
	  3. <input type="text" name="attribute1_3"  value="$attr1[3]" size="6"><br>
	  4. <input type="text" name="attribute1_4"  value="$attr1[4]" size="6"><br>
	  5. <input type="text" name="attribute1_5"  value="$attr1[5]" size="6"><br>
	  </td>
	  <td>$closedShop::font
	  Attribute 2:<br>
	  Attribute Name: <input type="text" name="attribute2_name"  value="$attr2[0]" size="10"><br>
	  <br>
	  1. <input type="text" name="attribute2_1"  value="$attr2[1]" size="6"><br>
	  2. <input type="text" name="attribute2_2"  value="$attr2[2]" size="6"><br>
	  3. <input type="text" name="attribute2_3"  value="$attr2[3]" size="6"><br>
	  4. <input type="text" name="attribute2_4"  value="$attr2[4]" size="6"><br>
	  5. <input type="text" name="attribute2_5"  value="$attr2[5]" size="6"><br>
	  </td>
	  <td>$closedShop::font
	  Attribute 3:<br>
	  Attribute Name: <input type="text" name="attribute3_name" value="$attr3[0]" size="10"><br>
	  <br>
	  1. <input type="text" name="attribute3_1"  value="$attr3[1]" size="6"><br>
	  2. <input type="text" name="attribute3_2"  value="$attr3[2]" size="6"><br>
	  3. <input type="text" name="attribute3_3"  value="$attr3[3]" size="6"><br>
	  4. <input type="text" name="attribute3_4"  value="$attr3[4]" size="6"><br>
	  5. <input type="text" name="attribute3_5"  value="$attr3[5]" size="6"><br>
	  
	  </td>
	  </tr></table>
	  
	  </td></tr>
	  
	  <tr bgcolor="$closedShop::mc"><th colspan="2">
	  $closedShop::font <font size="3"><input type="submit" value="Edit the Product"></th></tr>
	  </form></table><br><br>$closedShop::font<center>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="AdminScreen">
	  <input type="submit" value="Return to Control Panel"></form></center>
	  |;
	  closedShop::Footer();
	}
	elsif ($query->param("selected") == 1) {
	  my $cate = $query->param("category");
	  closedShop::Header();
	  print qq|
	  <form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="EditProduct">
	  <input type="hidden" name="selected" value="2">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="6">
	  $closedShop::font <font size="3"><font color="blue">Edit a product</font></th></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Edit</b></td>
	  <td>$closedShop::font<b>Category</b></td>
	  <td>$closedShop::font<b>Name</b></td>
	  <td>$closedShop::font<b>Code/Id</b></td>
	  <td>$closedShop::font<b>Price</b></td>
	  <td>$closedShop::font<b>Edit</b></td>
	  </tr>|;
	  
	  my $SQL = qq| select name,price,id,category,product_code from
	                $closedShop::sql_tables{"item"} where category="$cate"
	                order by name |;

	  my $sth = closedShop::DatabaseQuery($SQL);
 	  while (my ($name,$price,$id,$category,$code) = $sth->fetchrow_array()) {
		  print qq|
		    <tr bgcolor="$closedShop::mc">
			<td>$closedShop::font<input type="radio" name="id" value="$id"></td>
			<td>$closedShop::font $closedShop::categories{$category}</td>
			<td>$closedShop::font $name</td>
			<td>$closedShop::font $code</td>
			<td>$closedShop::font \$ $price</td>
			<td>$closedShop::font <input type="submit" value="Edit"></td>
			</tr>\n |;
		}
		
	  print qq|
        <tr bgcolor="$closedShop::mc"><td colspan="6"><center><input 
		type="submit" value="Edit"></td></tr>
		</table></form><br><br>$closedShop::font<center>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="AdminScreen">
	  <input type="submit" value="Return to Control Panel"></form></center>|;
	  closedShop::Footer();
	}
	else {
	  START:
	  closedShop::Header();
	  print qq|
	<form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="EditProduct">
	  <input type="hidden" name="selected" value="1">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	  $closedShop::font <font size="3"><font color="blue">Select Category</font></th></tr>
	 <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Category:</td>
	  <td>$closedShop::font <select name="category">  |;
	  foreach (sort keys %closedShop::categories) {
	    print qq|<option value="$_">$closedShop::categories{"$_"}</option>\n|;
	  }
	  print qq|
	  </select></td></tr>
	    <tr bgcolor="$closedShop::mc"><td colspan="2"><center><input 
		type="submit" value="Edit"></td></tr>
		</table></form><br><br>$closedShop::font<center>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="AdminScreen">
	  <input type="submit" value="Return to Control Panel"></form></center>|;
	  closedShop::Footer();
	}
}

###############################################################################
# FUNCTION:  ViewOrders
# DESCRIPTION: allows merchant to view / process orders
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ViewOrders
{
    my $mesg = shift;
	if ($query->param("selected") == 2) {
	  if ($query->param("process")) {
	    my $id = $query->param("id"); $id =~ tr/0-9//cd;

	    my $SQL = qq|update $closedShop::sql_tables{"order"} set 
	    date_processed=CURDATE() where id = "$id" |;
	    my $sth = closedShop::DatabaseQuery($SQL);
	    
		$SQL = qq|select user_id from $closedShop::sql_tables{"order"}  
	              where id = "$id" |;
	    $sth = closedShop::DatabaseQuery($SQL);
		my ($user_id) = $sth->fetchrow_array();
		
		$SQL = qq|select email,name,DATE_FORMAT(CURDATE(),"%W, %M %d %Y")
		          from $closedShop::sql_tables{"user"} where id = "$user_id" |;
	    $sth = closedShop::DatabaseQuery($SQL);		
		my ($email,$name,$now) = $sth->fetchrow_array();
		my $body = qq|
$name,

Your order (# $id) from $closedShop::CompanyName was 
processed and shipped on $now. 

For further information, please visit our website,
$closedShop::website
and log into your account or call us at $closedShop::CompanyPhone

Thank You,
$closedShop::CompanyName
$closedShop::CompanyEmail
|;
	    # send followup email to customer
	    closedShop::SendEmail($email,"Your order has been processed",$body);
		$query = new CGI({});
		return ViewOrders("You have successfully processed the order");
	  }
	  elsif ($query->param("viewold")) {
	    my $id = $query->param("id"); $id =~ tr/0-9//cd;
		if (!$id) { goto ListAllOldOrders; }
		closedShop::Header();
		
		DisplayIndividualOrder($id); 
	  	
		print qq|
		</table></form>\n
		<form action="$program_name" method="POST">
	    <input type="hidden" name="action" value="AdminScreen">
	    <input type="submit" value="Return to Control Panel"></form></center>|;

		closedShop::Footer();
	  }
	  else {
	    $query = new CGI({});
	    return ViewOrders();
	  }
	}
	elsif ($query->param("selected") == 1) {
	  if ($query->param("process")) {
	    my $id = $query->param("id"); $id =~ tr/0-9//cd;
		if (!$id) { goto ListAllOrders; }
		closedShop::Header();
		
		DisplayIndividualOrder($id);
	  	
		print qq|
		<tr bgcolor="$closedShop::mc">
        <td align="" colspan="2">$closedShop::font<div align="center">
		<input type="submit" name="process" value="Process Order"></div>
		</td></tr></table></form>
		<form action="$program_name" method="POST">
	    <input type="hidden" name="action" value="AdminScreen">
	    <input type="submit" value="Return to Control Panel"></form></center>|;
		closedShop::Footer();
	  }
		elsif ($query->param("deleteorder")) {
		  my $id = $query->param("id"); $id =~ tr/0-9//cd;
		  if (!$id) { goto ListAllOrders; }
		  my $SQL = qq|delete from $closedShop::sql_tables{"order"} 
			             where id = "$id" |;
	    my $sth = closedShop::DatabaseQuery($SQL);
			goto ListAllOrders;
		}
	  else {
	    ListAllOldOrders:
	    closedShop::Header();
	  	print qq|
	 	 $closedShop::font<font color="red">$mesg</font>
	 	 <form action="$program_name" method="POST" enctype="multipart/form-data">
	  	<input type="hidden" name="action" value="ViewOrders">
	  	<input type="hidden" name="selected" value="2">
	  	<table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="5">
	 	 $closedShop::font <font size="3"><font color="blue">Old Orders</font></th></tr>
	 	 <tr bgcolor="$closedShop::mc">
	  	<td>$closedShop::font<b>View Details</b></td>
	  	<td>$closedShop::font<b>Date</b></td>
	 	 <td>$closedShop::font<b>Name</b></td>
	 	 <td>$closedShop::font<b>Email</b></td>
	 	 <td>$closedShop::font<b>Total</b></td></tr>|;
	  
	 	 my $SQL = qq| select id,DATE_FORMAT(date_bought,"%W, %M %d %Y"),user_id,
	                total from $closedShop::sql_tables{"order"} 
	                where complete="Y" and date_processed is NOT NULL order by date_bought DESC |;

	 	 my $sth = closedShop::DatabaseQuery($SQL);
	 	 my $k = 0;
		 my $alltotal = 0;
 	 	 while (my ($oid,$date_bought,$uid,$total) = $sth->fetchrow_array()) {
	   	   $alltotal += $total;
		   my $SQL2 = qq| select name,email from $closedShop::sql_tables{"user"} 
	                where id="$uid" |;

	   	   my $sth2 = closedShop::DatabaseQuery($SQL2);
 	       my ($name,$email) = $sth2->fetchrow_array();
		   my $check = "checked";
		   if ($k != 0) { $check = ""; }
		   print qq|
		     <tr bgcolor="$closedShop::mc">
			 <td>$closedShop::font<input type="radio" name="id" value="$oid" $check></td>
			 <td>$closedShop::font $date_bought</td>
			 <td>$closedShop::font $name</td>
			 <td>$closedShop::font <a href="mailto:$email">$email</a></td>
			 <td>$closedShop::font \$ $total</td></tr>\n |;
		   $k++;
	  	 }
		
	    print qq|
        <tr bgcolor="$closedShop::mc"><td colspan="2">$closedShop::font<center><input 
		type="submit" name="viewold" value="View Details"></td>
		<td>&nbsp;</td><td><b>$closedShop::font Total: </td><td>$closedShop::font \$$alltotal</b></td>
		</tr>
		</table></form><br><br>$closedShop::font<center>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="AdminScreen">
	  <input type="submit" value="Return to Control Panel"></form></center>|;
	  closedShop::Footer();
	  }
	}
	else {
	  ListAllOrders:
	  closedShop::Header();
	  print qq|
	  $closedShop::font<font color="red">$mesg</font>
	  <form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="ViewOrders">
	  <input type="hidden" name="selected" value="1">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="5">
	  $closedShop::font <font size="3"><font color="blue">Process an Order</font></th></tr>
	  <tr bgcolor="$closedShop::mc">
	  <td>$closedShop::font<b>Process</b></td>
	  <td>$closedShop::font<b>Date</b></td>
	  <td>$closedShop::font<b>Name</b></td>
	  <td>$closedShop::font<b>Email</b></td>
	  <td>$closedShop::font<b>Total</b></td></tr>|;
	  
	  my $SQL = qq| select id,DATE_FORMAT(date_bought,"%W, %M %d %Y"),user_id,
	                total from $closedShop::sql_tables{"order"} 
	                where complete="Y" and date_processed is NULL order by date_bought |;

	  my $sth = closedShop::DatabaseQuery($SQL);
	  my $k = 0;
 	  while (my ($oid,$date_bought,$uid,$total) = $sth->fetchrow_array()) {
	    my $SQL2 = qq| select name,email from $closedShop::sql_tables{"user"} 
	                where id="$uid" |;

	    my $sth2 = closedShop::DatabaseQuery($SQL2);
 	    my ($name,$email) = $sth2->fetchrow_array();
		my $check = "checked";
		if ($k != 0) { $check = ""; }
		  print qq|
		    <tr bgcolor="$closedShop::mc">
			<td>$closedShop::font<input type="radio" name="id" value="$oid" $check></td>
			<td>$closedShop::font $date_bought</td>
			<td>$closedShop::font $name</td>
			<td>$closedShop::font <a href="mailto:$email">$email</a></td>
			<td>$closedShop::font \$ $total</td></tr>\n |;
		$k++;
	  }
		
	  print qq|
        <tr bgcolor="$closedShop::mc"><td colspan="2">$closedShop::font<center><input 
		type="submit" name="process" value="Process Order"><br><br>
    <input 
		type="submit" name="deleteorder" value="Remove Order"></td>
		<td>&nbsp;</td><td colspan="2">$closedShop::font<center><input 
		type="submit" name="viewold" value="View Processed Orders"></td>
		</tr>
		 <tr bgcolor="$closedShop::mc"><td colspan="5">$closedShop::font
		 By "Processing" an order, closedShop will send an email to the customer,
		 advising them that their order has been shipped. 
		 </td>
		</tr>
		</table></form><br><br>$closedShop::font<center>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="AdminScreen">
	  <input type="submit" value="Return to Control Panel"></form></center>|;
	  closedShop::Footer();
	}
}

###############################################################################
# FUNCTION:  DisplayIndividualOrder
# DESCRIPTION: info
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DisplayIndividualOrder
{
	my $id = shift;
	print qq|<form action="$program_name" method="POST">$closedShop::font
	    <input type="hidden" name="action" value="ViewOrders">
	    <input type="hidden" name="selected" value="2">
		<input type="hidden" name="id" value="$id">
		<table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	    $closedShop::font <font size="3"><font color="blue">Info for Order # $id </font></th></tr>
		
		<tr bgcolor="$closedShop::mc">
        <td >$closedShop::font
		<strong>Billing Info</strong><br>
		|;
		my $SQL	= qq| select b_name,b_address,b_city,b_state,
                      b_zip,b_phone,s_name,s_address,s_city,s_state,
                      s_zip,s_phone,item_ids,cc_name,cc,cc_num,cc_month,cc_year
					  from $closedShop::sql_tables{"order"} 
		              where id="$id"  |;
	    my $sth = closedShop::DatabaseQuery($SQL);
	    my ($b_name,$b_address,$b_city,$b_state,$b_zip,$b_phone,$s_name,$s_address,
		    $s_city,$s_state,$s_zip,$s_phone,$item_ids,$cc_name,$card,
			$cc_num,$cc_month,$cc_year) = $sth->fetchrow_array();
		
		print qq|
		$b_name<br>
		$b_address<br>
		$b_city, $b_state $b_zip<br>
		$b_phone
		</td>
		<td >$closedShop::font
		<strong>Shipping Info</strong><br>
		$s_name<br>
		$s_address<br>
		$s_city, $s_state $s_zip<br>
		$s_phone
		</td></tr>
		<tr bgcolor="$closedShop::mc">
        <td colspan="2">$closedShop::font
		<strong>Payment Info</strong><br>
		Credit Card: $card<br>
		Name on Card: $cc_name<br>
		Card Number: $cc_num<br>
		Exp. Date: $cc_month / $cc_year		
		</td></tr>
		<tr bgcolor="$closedShop::mc"><td colspan="2">
		$closedShop::font
		<b>Product Info</b>
		<center>
		<table width="100%" border="1" cellpadding="0" cellspacing="0">
		<tr bgcolor="$closedShop::mc">
		<td>$closedShop::font<strong>Name</strong></td>
		<td>$closedShop::font<strong>Code</strong></td>
		<td>$closedShop::font<strong>Quantity</strong></td>
		<td>$closedShop::font<strong>Attributes</strong></td>
		<td>$closedShop::font<strong>Price</strong></td>
		<td>$closedShop::font<strong>Shipping Price</strong></td>
		<td>$closedShop::font<strong>Total Cost</strong></td>
		</tr>
		|;
		
		my @ary = split(/~/,$item_ids);
		my $k = 1;	  
	    my $grand = 0;
		foreach (@ary) {
	  	  my ($iid,$qty,$attr) = split(/\|/,$_);
	  	  $SQL	= qq| select name,price,shipping_price,description,
	              category,product_code from $closedShop::sql_tables{"item"} where
				  id="$iid" |;
	      $sth = closedShop::DatabaseQuery($SQL);
	      my ($name,$price,$ship,$desc,$cat,$code) = $sth->fetchrow_array();
	  
	      my $total = sprintf("%.2f",$qty * ($price+$ship));
	      $grand += $total;

	      print qq|
	   	  <tr bgcolor="$closedShop::mc">
		  <td>$closedShop::font <a 
		  href="$closedShop::FileNames{item}?action=ShowItem&id=$iid">$name</a></td>
		  <td>$closedShop::font $code</td>
		  <td>$closedShop::font $qty</td>
		  <td>$closedShop::font $attr</td>
		  <td>$closedShop::font $qty x \$$price</td>
		  <td>$closedShop::font $qty x \$$ship</td>
		  <td>$closedShop::font \$ $total</td>
		  </tr>  |;	
	    }  
	
		my $tax = 0.00;
		if ($closedShop::TaxState eq uc($b_state)) {
		  $tax = sprintf("%.2f",$closedShop::TaxRate * $grand);
		}	
	
		my $total_cost = $tax + $grand;
		$total_cost = sprintf("%.2f",$total_cost);
		$grand = sprintf("%.2f",$grand);
	
		my $vrate = ($closedShop::TaxRate * 100);
	    print qq|<tr bgcolor="$closedShop::mc">
		<td colspan="5" align="right">&nbsp;</td><td>$closedShop::font
     	<strong>Sub Total: </td><td>$closedShop::font \$ $grand</strong></td></tr>
		<tr bgcolor="$closedShop::mc"><td colspan="5" align="right">&nbsp;</td>
		<td>$closedShop::font
     	<strong>Tax Total (for $closedShop::TaxState at $vrate%):</td><td>$closedShop::font
		 \$ $tax</strong></td></tr>
		<tr bgcolor="$closedShop::mc"><td colspan="5" align="right">&nbsp;</td><td>$closedShop::font
     	<strong>Grand Total (amount to charge):</td><td>$closedShop::font 
		<strong>\$ $total_cost</strong></strong></td></tr>
		</table></td></tr>|;
}

###############################################################################
# FUNCTION:  DeleteProduct
# DESCRIPTION: edit a product
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DeleteProduct
{
	if ($query->param("selected") == 2) {
	  my $id = $query->param("id"); $id =~ tr/0-9//cd;
	  if ($id == 0 || !$id) {
	    goto START;
	  }
	  my $SQL = qq| delete from $closedShop::sql_tables{"item"} where id = "$id" |;
	  my $sth = closedShop::DatabaseQuery($SQL);
 	    
	  return AdminScreen("You have successfully deleted the product");
	}
	elsif ($query->param("selected") == 1) {
	  my $cate = $query->param("category");
	  closedShop::Header();
	  print qq|
	  <form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="DeleteProduct">
	  <input type="hidden" name="selected" value="2">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="6">
	  $closedShop::font <font size="3"><font color="blue">Delete a product</font></th></tr>
	  <tr bgcolor="$closedShop::mc">
	  <td>$closedShop::font<b>Delete</b></td><td>$closedShop::font<b>Category</b></td>
	  <td>$closedShop::font<b>Name</b></td>
	  <td>$closedShop::font<b>Code/Id</b></td>
	  <td>$closedShop::font<b>Price</b></td>
	  <td>$closedShop::font<b>Delete</b></td>
	  </tr>|;
	  
	  my $SQL = qq| select name,price,id,category,product_code from $closedShop::sql_tables{"item"} 
	                where category = "$cate" order by name |;

	  my $sth = closedShop::DatabaseQuery($SQL);
 	  while (my ($name,$price,$id,$category,$code) = $sth->fetchrow_array()) {
		  print qq|
		    <tr bgcolor="$closedShop::mc">
			<td>$closedShop::font<input type="radio" name="id" value="$id"></td>
			<td>$closedShop::font $closedShop::categories{$category}</td>
			<td>$closedShop::font $name</td>
			<td>$closedShop::font $code</td>
			<td>$closedShop::font \$ $price</td>
			<td>$closedShop::font<input type="submit" value="Delete"></td>
			</tr>\n |;
		}
		
	  print qq|
        <tr bgcolor="$closedShop::mc"><td colspan="6"><center><input 
		type="submit" value="Delete"></td></tr>
		</table></form><br><br>
		$closedShop::font<center>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="AdminScreen">
	  <input type="submit" value="Return to Control Panel"></form></center>|;
	  closedShop::Footer();
	}
	else {
	  START:
	  closedShop::Header();
	  print qq|
	<form action="$program_name" method="POST" enctype="multipart/form-data">
	  <input type="hidden" name="action" value="DeleteProduct">
	  <input type="hidden" name="selected" value="1">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	  $closedShop::font <font size="3"><font color="blue">Select Category</font></th></tr>
	 <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font Category:</td>
	  <td>$closedShop::font <select name="category">  |;
	  foreach (sort keys %closedShop::categories) {
	    print qq|<option value="$_">$closedShop::categories{"$_"}</option>\n|;
	  }
	  print qq|
	  </select></td></tr>
	    <tr bgcolor="$closedShop::mc"><td colspan="2"><center><input 
		type="submit" value="Choose Items to Remove"></td></tr>
		</table></form><br><br>$closedShop::font<center>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="AdminScreen">
	  <input type="submit" value="Return to Control Panel"></form></center>|;
	  closedShop::Footer();
	}
}

###############################################################################
# FUNCTION:  ChangeVariables
# DESCRIPTION: change variables of the program
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ChangeVariables
{
	my ($session,$uid) = closedShop::GetSessionCookie();
	if ($query->param("selected") == 1) {
 	  open(FILE,">$closedShop::varsfile");
	  foreach ($query->param()) {
	    my @var = $query->param("$_");
	    my $var = join(",",@var); $var =~ s/~/*/g;
	    print FILE qq|$_~$var\n|; 
	  }
	  close(FILE); 
	  return AdminScreen("You have successfully updated your program variables");
	}
	else {
	  my %hash = ();
	  open(FILE,"$closedShop::varsfile");
	  while (<FILE>) {
	    chomp($_);
	    my @ary = split(/~/,$_);
	    $hash{$ary[0]} = $ary[1];
	  }
	  close(FILE);	
	  closedShop::Header();
	  print qq|<br>
	
	<table width="530" border="0"><tr><td colspan="2" bgcolor="$closedShop::tc">
	$closedShop::font <font color="blue"><b>Installation - Company Info</b></font></td></tr>
	<form action="$program_name" method="POST" enctype="multipart/form-data">
	<input type="hidden" name="action" value="ChangeVariables">
	<input type="hidden" name="selected" value="1">
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Company Name</b>: </font></td><td>$closedShop::font 
	<input type="text" name="CompanyName" value="$hash{CompanyName}" size="20"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Company Email</b>: </font></td><td>$closedShop::font 
	<input type="text" name="CompanyEmail" value="$hash{CompanyEmail}" size="20"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Company Phone</b>: </font></td><td>$closedShop::font
	<input type="text" name="CompanyPhone" value="$hash{CompanyPhone}" size="20"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Company Address</b>: </font></td><td>$closedShop::font 
	<input type="text" name="CompanyAddress" value="$hash{CompanyAddress}" size="40"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Company City</b>: </font></td><td>$closedShop::font 
	<input type="text" name="CompanyCity" value="$hash{CompanyCity}" size="40"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Company State</b>: </font></td><td>$closedShop::font 
	<input type="text" name="CompanyState" value="$hash{CompanyState}" size="20"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Company Zip</b>: </font></td><td>$closedShop::font 
	<input type="text" name="CompanyZip" value="$hash{CompanyZip}" size="20"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Tax Rate for State</b>: </font></td><td>$closedShop::font 
	<input type="text" name="TaxRate" value="$hash{TaxRate}" size="5"></td></tr>
	 
	 	 	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Minumum Amount for order:</b><br>
   (leave blank for no min.) </font></td><td>$closedShop::font \$ 
	<input type="text" name="MinimumOrder" value="$hash{MinimumOrder}" size="5"></td></tr>
	 
	 
	 <tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Select Accepted Credit Cards</b>:</td><td>$closedShop::font 
	  |;
	  
	  my ($mcheck,$vcheck,$dcheck,$acheck,$dicheck);
	  foreach (keys %closedShop::AcceptedCards) {
	    if ($_ eq "mastercard") { $mcheck = "checked"; }
		if ($_ eq "visa") { $vcheck = "checked"; }
		if ($_ eq "discover") { $dcheck = "checked"; }
		if ($_ eq "amex") { $acheck = "checked"; }
		if ($_ eq "diner") { $dicheck = "checked"; }
	  }
	  
	  print qq|
	<input type="checkbox" $mcheck name="creditcard" value="mastercard">Mastercard<br>
	<input type="checkbox" $vcheck name="creditcard" value="visa">Visa<br>
	<input type="checkbox" $dcheck name="creditcard" value="discover">Discover<br>
	<input type="checkbox" $acheck name="creditcard" value="amex">American Express<br>
	<input type="checkbox" $dicheck name="creditcard" value="diner">Diner's Club<br>
	
	 </td></tr>
	 
	 <tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Create Categories</b>:<br>
	 <i>Create as many as you need</i></td><td>$closedShop::font |;
	  
	for (my $j = 1; $j < 13; $j++) {
	  my $cate = $hash{"category$j"};
	  print qq|Category $j: <input type="text" name="category$j" value="| . $cate
	   . qq|" size="20"><br>\n|;
	 }
	
	print qq|</td></tr></table> <br><br>|;
	
	if ($uid == 1) {
			print qq|
	
	 	
	 <table width="530" border="0"><tr><td colspan="2" bgcolor="$closedShop::tc">
	$closedShop::font <font color="blue"><b>Installation - SQL Database Info</b></font></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Database Name</b>: </font></td><td>$closedShop::font 
	<input type="text" name="DatabaseName" value="$hash{DatabaseName}" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Database Username</b>: </font></td><td>$closedShop::font 
	<input type="text" name="DatabaseUsername" value="$hash{DatabaseUsername}" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Database Password</b>: </font></td><td>$closedShop::font 
	<input type="password" name="DatabasePassword" value="$hash{DatabasePassword}" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Database Location</b><br>
	<i>Only change if you know what you are doing</i>: </font></td><td>$closedShop::font 
	<input type="text" name="DatabaseLocation" value="$hash{DatabaseLocation}" size="20"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Database Table Names</b>: </font></td><td>$closedShop::font 
	Admin Table: <input type="text" name="admintable" value="$hash{admintable}" size="20"><br>
	Item Table: <input type="text" name="itemtable" value="$hash{itemtable}" size="20"><br>
	Order Table: <input type="text" name="ordertable" value="$hash{ordertable}" size="20"><br>
	User Table: <input type="text" name="usertable" value="$hash{usertable}" size="20"><br>
	
	</td></tr>
		 </table><br><br>
	
	 <table width="530" border="0"><tr><td colspan="2" bgcolor="$closedShop::tc">
	$closedShop::font <font color="blue"><b>Installation - Server Specifics</b></font></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Website URL</b>: <br>
	No Trailing Slash</font></td><td>$closedShop::font 
	<input type="text" name="website" value="$hash{website}" size="30"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>URL to closedShop Folder</b>: <br>
	No Trailing Slash</font></td><td>$closedShop::font 
	<input type="text" name="urltofolder" value="$hash{urltofolder}" size="30"></td></tr>
	
		<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>URL to Continue Shopping Link</b>:<br>
  (If blank, will return to search page) <br>
	No Trailing Slash</font></td><td>$closedShop::font 
	<input type="text" name="ContinueShoppingURL" value="$hash{ContinueShoppingURL}" size="30"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>URL to Final link after completed order</b>:<br>
  (If blank, will return to homepage of website) <br>
	No Trailing Slash</font></td><td>$closedShop::font 
	<input type="text" name="CompletedOrderURL" value="$hash{CompletedOrderURL}" size="30"></td></tr>
	
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Server Path to Image Folder</b>: <br>
	No Trailing Slash</font></td><td>$closedShop::font 
	<input type="text" name="IMAGEPATH" value="$hash{IMAGEPATH}" size="30"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>URL to Image Folder</b>: <br>
	No Trailing Slash</font></td><td>$closedShop::font 
	<input type="text" name="IMAGEURL" value="$hash{IMAGEURL}" size="30"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>URL to <b>SECURE</b> Image Folder</b>: <br>
	No Trailing Slash</font></td><td>$closedShop::font 
	<input type="text" name="SecureImageURL" value="$hash{SecureImageURL}" size="30"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Path to Header File</b>: </font></td><td>$closedShop::font 
	<input type="text" name="header" value="$hash{header}" size="30"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Path to Footer File</b>: </font></td><td>$closedShop::font 
	<input type="text" name="footer" value="$hash{footer}" size="30"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Mail Program</b>: </font></td><td>$closedShop::font 
	<input type="text" name="mailprog" value="$hash{mailprog}" size="30"></td></tr>
	
	</table><br><br>
	|;
		}
		else {
			print qq|
	<input type="hidden" name="DatabaseName" value="$hash{DatabaseName}">
	<input type="hidden" name="DatabaseUsername" value="$hash{DatabaseUsername}">
	<input type="hidden" name="DatabasePassword" value="$hash{DatabasePassword}">
	<input type="hidden" name="DatabaseLocation" value="$hash{DatabaseLocation}">
	<input type="hidden" name="admintable" value="$hash{admintable}">
	<input type="hidden" name="itemtable" value="$hash{itemtable}">
	<input type="hidden" name="ordertable" value="$hash{ordertable}">
	<input type="hidden" name="usertable" value="$hash{usertable}">
	<input type="hidden" name="website" value="$hash{website}">
	<input type="hidden" name="urltofolder" value="$hash{urltofolder}">
	<input type="hidden" name="SecureImageURL" value="$hash{SecureImageURL}">
	<input type="hidden" name="IMAGEFOLDER" value="$hash{IMAGEFOLDER}">
	<input type="hidden" name="IMAGEPATH" value="$hash{IMAGEPATH}">
	<input type="hidden" name="IMAGEURL" value="$hash{IMAGEURL}">
	<input type="hidden" name="header" value="$hash{header}">
	<input type="hidden" name="footer" value="$hash{footer}">
	<input type="hidden" name="mailprog" value="$hash{mailprog}">
			|;
		}

		print qq|
	
	 <table width="530" border="0"><tr><td colspan="2" bgcolor="$closedShop::tc">
	$closedShop::font <font color="blue"><b>Installation - Extras</b></font></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Table Top Colors</b>: </font></td><td>$closedShop::font 
	<input type="text" name="tc" value="$hash{tc}" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Table Main Colors</b>: </font></td><td>$closedShop::font 
	<input type="text" name="mc" value="$hash{mc}" size="20"></td></tr>
	
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Font</b>: </font></td><td>$closedShop::font 
	<input type="text" name="font" value="$hash{font}" size="30"></td></tr>

	</table><br><br>
	
	|;
	
	if($uid == 1) {
			print qq|

	<table width="400" border="0"><tr><td colspan="2" bgcolor="$closedShop::tc">
	$closedShop::font <font color="blue"><b>Installation - File names</b></font></td></tr>

	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>URL for Admin.pl</b>: </font></td><td>$closedShop::font 
	<input type="text" name="FileName_admin" value="$hash{FileName_admin}" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>URL for Cart.pl</b>: </font></td><td>$closedShop::font 
	<input type="text" name="FileName_cart" value="$hash{FileName_cart}" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>URL for User.pl</b>: </font></td><td>$closedShop::font 
	<input type="text" name="FileName_user" value="$hash{FileName_user}" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>URL for Item.pl</b>: </font></td><td>$closedShop::font 
	<input type="text" name="FileName_item" value="$hash{FileName_item}" size="20"></td></tr>

	</table><br><br>|;
		}
		else {
			print qq|
	<input type="hidden" name="FileName_admin" value="$hash{FileName_admin}">
	<input type="hidden" name="FileName_cart" value="$hash{FileName_cart}">
	<input type="hidden" name="FileName_user" value="$hash{FileName_user}">
	<input type="hidden" name="FileName_item" value="$hash{FileName_item}">
			|;
		}

		print qq|

	
	 <table width="530" border="0"><tr><td colspan="2" bgcolor="$closedShop::tc"><center>
	$closedShop::font <font color="blue"><b><input type="submit" value="Edit Variables"></b></font></td></tr>
	</table></form>
	 
	</center></div>
	$closedShop::font<center>
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="AdminScreen">
	  <input type="submit" value="Return to Control Panel"></form></center>|;
	  closedShop::Footer();
	}
}

###############################################################################
# FUNCTION:  Login
# DESCRIPTION: The user will log in, sending the username and password
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Login
{
	my $username = closedShop::ValidateInput($query->param('username'));
	my $password = closedShop::ValidateInput($query->param('password'));
	
	my $SQL	= qq| select id from $closedShop::sql_tables{"admin"} where username = "$username" and
	              password = "$password" |;
	my $sth = closedShop::DatabaseQuery($SQL);
	my ($uid) = $sth->fetchrow_array();
	
	# if we pass this condition, the user has logged in, so retrieve information
	if (!$uid) {	
		return LoginScreen("You supplied an incorrect user/password combination");  
	}
	else {	
	  LoginTheAdmin($uid);
	}
}

################################################################################
# FUNCTION:  AdminScreen
# DESCRIPTION: Displays the admin screen
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub AdminScreen
{
	my $message = shift;
	closedShop::Header($message);
	$message = "" if ($message eq "nohttp");
	
	my $date = localtime;
	print qq|<br><center>
	$closedShop::font	Welcome back to the administration control panel.<br>
	Today's date is: $date<br>
	<font color="red">$message</font><br>$closedShop::font
	<table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="1">
	$closedShop::font <font size="3"><font color="blue">Administration Options</font></th></tr>
	|;
	
	foreach (sort keys %actions) {
	  unless ($_ eq "AdminScreen") {
	    print qq|
	    <form action="$program_name" METHOD="POST">
	    <input type="hidden" name="action" value="$_">
	    <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font 
		<input type="submit" value="$actions{$_}">
	    </center></td></tr></form>|;
	  }
	}
	print qq|</table></center></div>|;
	
	closedShop::Footer();
}

################################################################################
# FUNCTION:  LoginScreen
# DESCRIPTION: login screen for admin
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub LoginScreen
{
    my $mesg = shift;
	closedShop::Header();
	print qq| <form action="$program_name" method="POST">
	<input type="hidden" name="action" value="Login">
	$closedShop::font
	<font color="red">$mesg</font>
	<table border="0"><tr bgcolor="$closedShop::tc"><td colspan="2">$closedShop::font<font color="blue">
	<div align="center"><b>Login</b></div></td></tr><tr bgcolor="$closedShop::mc"><td>$closedShop::font
	Username: </td><td><input type="text" name="username" size="10"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font
	Password: </td><td><input type="password" name="password" size="10"></td></tr>
	<tr bgcolor="$closedShop::mc"><td  colspan="2">
	<div align="center"><input type="submit" name="admin_login" value="Login"></div>
	</td></tr></table>
	<br></form>
	|;
	
	closedShop::Footer();
}

################################################################################
# FUNCTION:  LoginTheAdmin
# DESCRIPTION: process screen
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub LoginTheAdmin
{
      my $uid = shift;
	  my $session = closedShop::CreateSession();
	  my $SQL = qq| update $closedShop::sql_tables{"admin"} set session="$session" where id = "$uid" |;
	  my $sth = closedShop::DatabaseQuery($SQL);
	  closedShop::SetSessionCookie($session,$uid);
	  AdminScreen("nohttp");
}









