#!/usr/bin/perl

package CallCGI;

##########################################################################################
##########################################################################################
#  Program Name: CallCGI.pm                                                              #
#        Author: Joshua Freeman (frejos@perlConcepts.com)                                #
#    On the Web: http://www.perlConcepts.com/                                            #
#   Description: Transparently redirects form POST or GET information to a               # 
#                second CGI or ASP program.  Allowing the first program to manipulate    #
#                the data as needed.  The output of the second CGI program can           #
#                be transparently sent as output.                                        #
#     Copyright: (c)1998-2001 Joshua Freeman, all rights reserved.                       #
#       License: By using this program you agree to the End User License Agreement       #
#                located online at http://www.perlconcepts.com/programs/cgiwrap/eula.asp #
#                Currently this program may be used for FREE, but all copyright          #
#                information and links to perlConcepts MUST remain intact.               #
#     Questions: Email your questions, comments, and suggestions to                      #
#                CallCGI@perlConcepts.com                                                #
##########################################################################################
##########################################################################################

##########################################################################################
##########################################################################################
#  Revision History                                                                      #
##########################################################################################
##########################################################################################

$VERSION = '1.10';

# 1.10    (20010414) : Converted to Module and upgraded functionality
#         (20010407) : Prepared for distribution
# 1.00    (19980908) : Original Created

##########################################################################################
##########################################################################################
#  PROGRAM BEGINS -- DO NOT EDIT BELOW THIS LINE                                         #
##########################################################################################
##########################################################################################

sub new {
   my $class = shift;

   my %m_Vars;

   ($m_Vars{'URL'}, $m_Vars{'Data'}) = @_;
   
   my $rh_m_Vars = \%m_Vars;
   bless $rh_m_Vars, $class;
   return $rh_m_Vars;
}

sub URL {
   my $self = shift;
   my $URL = shift;
   my $oldURL = $self->{'URL'};

   $self->{'URL'} = $URL if $URL ne "";

   return $oldURL;
}

sub Data {
   my $self = shift;
   my $Data = shift;
   my $oldData = $self->{'Data'};

   $self->{'Data'} = $Data if $Data ne "";

   return $oldData;
}

sub CallCGI {
   my $self = shift;
   my ($URL, $Data) = @_;

   $URL = $self->URL($URL);
   $Data = $self->Data($Data);

   my @RETVAL = $self->m_CallCGI($URL, $Data);

   $self->{'Status'} = $RETVAL[0];
   $self->{'Document'} = $RETVAL[1];

   return @RETVAL;
}

sub Status { return $_[0]->{'Status'}; }
sub Document { return $_[0]->{'Document'}; }

sub AddValue {
   my $self = shift;
   my ($Key, $Val) = @_;
   $self->{'Data'} .= "&" if $self->{'Data'} ne "";
   $self->{'Data'} .= $self->URLEncode($Key) . "=" . $self->URLEncode($Val);
}

sub AddHash {
   my $self = shift;
   my (%Hash) = @_;

   for (keys(%Hash)) {
      $self->AddValue($_, $Hash{$_});
   }
}

sub TakeEnv {
   my $self = shift;
   my $PostData;

   if ($ENV{'CONTENT_LENGTH'} > 0) {
      read (STDIN, $PostData, $ENV{'CONTENT_LENGTH'});
   }
   else {
      $PostData = $ENV{'QUERY_STRING'};
   }

   $self->{'Data'} .= "&" if $self->{'Data'} ne "";
   $self->{'Data'} .= $PostData;
}

sub DebugInfo {
   my $self = shift;

   my $retval;

   for (keys(%$self)) {
      $retval .= "  $_ = $self->{$_}\n";
   }
   return $retval;
}

sub m_CallCGI {
   my $self = shift;

   #######################################################################################
   #######################################################################################
   # Localize Variables                                                                  #
   #######################################################################################
   #######################################################################################

   my ($URL, $hostname, $document, $name, $aliases, $type, $len, $addr);
   my ($socketaddr, $PostData, $ContentLength, $NewPage, $in, $Header);
   my ($old);

   #######################################################################################
   #######################################################################################
   # Get Arguments                                                                       #
   #######################################################################################
   #######################################################################################

   ($URL, $PostData) = @_; #ARGS passed to sub

   #Now we split the URL into its parts
   ($hostname, $document) = split (/\//, $URL, 2);

   # Split takes out the value '/', so we have to put it back
   $document = "/" . $document;

   #If we have $PostData as an argument, we do not take the data from our 
   #environment, IOW this argument bypasses all other POST or GET information.
   #I use $TakeEnv as a boolean variable for this purpose.
   $TakeEnv = ($PostData eq "");

   #######################################################################################
   #######################################################################################
   # Find new Content-Length                                                             #
   #######################################################################################
   #######################################################################################

   $ContentLength = int(length($PostData));

   #######################################################################################
   #######################################################################################
   # Create Socket                                                                       #
   #######################################################################################
   #######################################################################################

   ($name, $aliases, $type, $len, $addr) = gethostbyname($hostname);
   # socket(FILEHANDLE, AF_INET, SOCK_STREAM, ?)    
   socket (CGI, 2, 1, 0); 
   # pack(template, AF_INET, PORT, $addr)
   $socketaddr = pack ('S n a4 x8', 2, 80, $addr); 
   connect (CGI, $socketaddr);

   #######################################################################################
   #######################################################################################
   # Prepare File Handle                                                                 #
   #######################################################################################
   #######################################################################################

   $| = 1; #No Buffering
   $old = select(CGI); #Move CGI to default
   $| = 1; #No Buffering
   select($old); #Restore default

   #######################################################################################
   #######################################################################################
   # Send Request to CGI                                                                 #
   #######################################################################################
   #######################################################################################

   print CGI "POST $document HTTP/1.0\n";
   print CGI "Host: $hostname\n";
   print CGI "Content-Length: $ContentLength\n";
   print CGI "User-Agent: CallCGI/$VERSION ( http://perlconcepts.com/programs/CallCGI/; CallCGI\@perlConcepts.com )\n";
   print CGI "Referer: $ENV{'HTTP_REFERER'}\n";
   print CGI "Content-Type: application/x-www-form-urlencoded\n";
   print CGI "\n"; #End Headers

   if ($ContentLength > 0) {
      print CGI $PostData;
      print CGI "\n"; #End Data
   }

   #######################################################################################
   #######################################################################################
   # Get Response from CGI                                                               #
   #######################################################################################
   #######################################################################################

   while (read (CGI, $in, 1024) == 1024) {
      $NewPage .= $in;
   }
   $NewPage .= $in;


   #######################################################################################
   #######################################################################################
   # Close Socket                                                                        #
   #######################################################################################
   #######################################################################################

   close(CGI);

   #######################################################################################
   #######################################################################################
   # Return Variables as Array                                                           #
   #######################################################################################
   #######################################################################################

   return split (/\r?\n\r?\n/, $NewPage, 2);
}

sub URLEncode {
   my $self = shift;
   my ($temp) = @_;
   my $retval;

   foreach (split //, $temp) {
      if ( $_ eq " ") {
         $retval .= "+";
         next;
      }

      if (ord($_) < 0x41 || ord($_) > 0x7a) { 
         $retval .= sprintf("%%%02x",ord($_));
      }
      else { 
         $retval .= $_;
      }
   }
   return  $retval;
}

sub URLDecode {
   my $self = shift;
   my ($temp) = @_;

   $temp =~ s/\+/ /g;
   $temp =~ s/%(..)/pack("c",hex($1))/ge;
   return  $temp;
}


####
1; # Library Include
####
