#!/usr/bin/perl 

################################################################################
# 
# PROGRAM:  Cart.pl - from closedShop - Open Shopping Cart 
#
# RCSID:  $Id$
#
# DESCRIPTION: Shopping cart part
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:  
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-05-03   cbf    Created
# 2001-06-07   cbf    Added attribute support for items
# 2001-06-19   cbf    Added credit card validation
# 2001-06-24   cbf    If you try and retrieve the value of a nonexistent cookie
#          	            netscape doesn't like it. netscape sucks.
# 2001-08-04   cbf    Continue Shopping Link now goes to last item added to cart
# 2001-08-05   cbf    Was missing Header call for an error message
#                     If you're logged in already, checkout process doesn't 
#                        ask for username/password
# 2001-08-12   cbf    All input validated
# 2001-10-28   cbf    fixed shipping,billing errors
# 2002-05-14   cbf    now saves billing address info
# 2002-05-14   cbf    Continue Shopping and Final Order link variables added
################################################################################

#use CGI::Carp(fatalsToBrowser);
use closedShop;
use CGI qw/:standard/;
use CGI::Cookie;
use strict;
use CGI;
use DBI;
use Fcntl ':flock';

closedShop::Initialize();

###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query = new CGI;
my $program_name = "$closedShop::FileNames{cart}";
	
my %actions = (AddProduct => "Add Product",
			   DeleteProduct => "Delete Product",
			   ViewCart => "View Your Shopping Cart",
			   Checkout => "Checkout",
			   AddWishList => "Add Item to Wishlist"
			   );
################################################################################


Main();
closedShop::Cleanup();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main
{
	my $action = $query->param('action');
	$action =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	if (!$actions{$action}) { 
	  ViewCart("");
	}
	else {
	  eval ($action);
	}
}

###############################################################################
# FUNCTION:  AddWishList
# DESCRIPTION: add to wishlist
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub AddWishList
{
	my $id = $query->param("id"); $id =~ tr/0-9//cd;
	
	my ($session,$uid) = closedShop::GetSessionCookie();
	if (!closedShop::Authenticate($session,"user")) {
	  print "Location: $closedShop::FileNames{user}?message=Please%20login%20or%20create%20an%20account%20to%20use%20your%20wishlist\n\n";
	}

	closedShop::Header();
	my $SQL = qq|select wishlist_ids from $closedShop::sql_tables{"user"} where id="$uid"|;
	my $sth = closedShop::DatabaseQuery($SQL); 	
	my ($wish) = $sth->fetchrow_array();
	
	if ($id) {
	  if ($wish !~ /$id/) {
	    $wish .= $id . qq|:|;
	  }
	  else {
	    $wish = $wish;
	  }
	}
	
	$SQL = qq|update $closedShop::sql_tables{"user"} 
	set wishlist_ids = "$wish" where id="$uid"|;
	$sth = closedShop::DatabaseQuery($SQL); 	
	
	print qq|
	 <table border="0" width="100%"><tr><td colspan="6" bgcolor="$closedShop::tc">
	 $closedShop::font <font color="blue"><b><a name="Wishlist">WishList</a></b></font></td></tr>
	 <tr bgcolor="$closedShop::mc">
	 <td>$closedShop::font<b>Item Name</b></td>
	 <td>$closedShop::font<b>Category</b></td>
	  <td>$closedShop::font<b>Shipping Price</b></td>
	  <td>$closedShop::font<b>Item Price</b></td>
	  <td>$closedShop::font<b>Add to Cart</b></td>
	  <td>$closedShop::font<b>Remove from Wishlist</b></td>
	  </tr>|;
	  
	my @wish = split(/:/,$wish);

	foreach (@wish) {
	  my $SQL2	= qq| select name,category,shipping_price,
	  price from $closedShop::sql_tables{"item"} where id="$_" |;
	  my $sth2 = closedShop::DatabaseQuery($SQL2);
	  my ($n,$c,$s,$p) = $sth2->fetchrow_array();
	  print qq|
		<tr bgcolor="$closedShop::mc">
	 	<td>$closedShop::font<a href="$closedShop::FileNames{item}?action=ShowItem&id=$_">$n</a></td>
	 	<td>$closedShop::font $closedShop::categories{$c}</td>
	  	<td>$closedShop::font \$$s</td>
	  	<td>$closedShop::font \$$p</td>
	  	<td>$closedShop::font 
		<a href="$closedShop::FileNames{cart}?action=AddProduct&id=$_&quantity=1">Add to Cart</a></td>
		<td>$closedShop::font 
		<a href="$closedShop::FileNames{user}?action=Wishlist&Remove=1&id=$_">Remove from Wishlist</a></td></tr>
		  |;
	 }
	 
	 print qq|</table>|;
	
	closedShop::Footer();
}

###############################################################################
# FUNCTION:  AddProduct
# DESCRIPTION: add a product to users shopping cart
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub AddProduct
{
	my $id = $query->param("id"); $id =~ tr/0-9//cd;
	my $quantity = $query->param("quantity"); $quantity =~ tr/0-9//cd;	
	
	my $SQL	= qq| select attributes from $closedShop::sql_tables{"item"} where
				  id="$id" |;
	my $sth = closedShop::DatabaseQuery($SQL);
	my ($attr) = $sth->fetchrow_array();
	my @attr = split(/\|/,$attr); my @attr1 = split(/~/,$attr[0]);
	my @attr2 = split(/~/,$attr[1]); my @attr3 = split(/~/,$attr[2]);
	 
	my $attribute1 = $query->param("attribute1"); $attribute1 =~ tr/a-zA-Z0-9-_ -//cd;	 
	my $attribute2 = $query->param("attribute2"); $attribute2 =~ tr/a-zA-Z0-9-_ -//cd;	 
	my $attribute3 = $query->param("attribute3"); $attribute3 =~ tr/a-zA-Z0-9-_ -//cd;	 
	
	if ($attr1[0] && !$attribute1) { $attribute1 = $attr1[1]; }
	if ($attr2[0] && !$attribute2) { $attribute2 = $attr2[1]; }
	if ($attr3[0] && !$attribute3) { $attribute3 = $attr3[1]; }
	
	my $attribute = $attribute1 . "?" . $attribute2 . "?" . $attribute3;
	
	if (!$id || !$quantity) {
	  return ViewCart();
	}
	
    my %cookies = fetch CGI::Cookie;
    my $cart = "";
	if ($cookies{'cart'}) {
	  $cart = $cookies{'cart'}->value if (%cookies);
	}

	$cart .= qq@$id|$quantity|$attribute~@;
		
	my $cookie = new CGI::Cookie(-name=>'cart',-value=>"$cart");
	my $cookie2 = new CGI::Cookie(-name=>'lastitem',-value=>"$id");

	print header(-Cookie=>[$cookie,$cookie2],
				 -type => "text/html"
				 );
	
	print qq|
	<html><head><meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
    <meta http-equiv="refresh" content="0; URL=$program_name">
    <title></title></head><body>&nbsp;</body></html> |;

}

###############################################################################
# FUNCTION:  Checkout
# DESCRIPTION: checkout process
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Checkout
{
	my $message = shift;
	
	my %cookies = fetch CGI::Cookie;
    my $cart = $cookies{'cart'}->value if ($cookies{'cart'});
	if (!$cart) {
	  return ViewCart();
	}
	
	if ($query->param("selected") == 4) {
	  my %cookies = fetch CGI::Cookie;
	  my $oid = $cookies{'oid'}->value if ($cookies{'oid'});
	  
	  my ($session,$uid) = closedShop::GetSessionCookie();
	  if (!closedShop::Authenticate($session,"user")) {
	      closedShop::Header();
	      print qq|$closedShop::font<font color="red">You are not
		  logged in properly</font>|;
		  DISPLAY();
		  return closedShop::Footer();
	  }
	
	  my $SQL = qq|update $closedShop::sql_tables{"order"} set date_bought=CURDATE(),
	    complete="Y" where id = "$oid" and user_id="$uid"|;
	  my $sth = closedShop::DatabaseQuery($SQL); 	
	
	my $receiptbody = qq|Name\tQty\tAttributes\tPrice\tShipping Price\tTotal Cost\n\n|;
	
	 $SQL = qq|select item_ids,b_state from $closedShop::sql_tables{"order"} 
	 where id = "$oid" and user_id="$uid"|;
	 $sth = closedShop::DatabaseQuery($SQL); 		
	 my ($item_ids,$b_state) = $sth->fetchrow_array();
	 
	  my @ary = split(/~/,$item_ids);
	  my $k = 1; my $grand = 0;
	  foreach (@ary) {
	    my ($id,$qty,$attr) = split(/\|/,$_);
		my @attr = split(/\?/,$attr); $attr = join("<br>",@attr);
	  	$SQL	= qq| select name,price,shipping_price,description,
	              category from $closedShop::sql_tables{"item"} where
				  id="$id" |;
	    $sth = closedShop::DatabaseQuery($SQL);
	    my ($name,$price,$ship,$desc,$cat) = $sth->fetchrow_array();
	  
	    my $total = sprintf("%.2f",$qty * ($price+$ship));
	    $grand += $total;
		if (length($name) > 18) { $name = substr($name,0,18) . "..."; }
	    $receiptbody .= qq|$name\t$qty\t$attr\t$qty x \$$price\t$qty x \$$ship\t\$ $total\n|;	
	  }  
	
		my $tax = 0.00;
		if ($closedShop::TaxState eq uc($b_state)) {
		  $tax = sprintf("%.2f",$closedShop::TaxRate * $grand);
		}	
		my $total_cost = $tax + $grand;
		$total_cost = sprintf("%.2f",$total_cost);
		$grand = sprintf("%.2f",$grand);
	
		$receiptbody .= qq|\nSub Total: \$$grand\nTotal Tax: \$$tax\nTotal Cost: \$$total_cost\n|;
	
	    $SQL = qq|select name,email from $closedShop::sql_tables{"user"} where id="$uid"|;
	    $sth = closedShop::DatabaseQuery($SQL); 	
	    my ($un,$ue) = $sth->fetchrow_array();
	
	    my $body = qq|
$un,

Thank you for purchasing from $closedShop::CompanyName

Order Details
----------------------------
$receiptbody
----------------------------

To view the status of your order, please visit our website,
$closedShop::website
and log into your account. 

You will be contacted when your order is fulfilled.

Thank You,
$closedShop::CompanyName
$closedShop::CompanyEmail
|;

	my $adminurl = $closedShop::FileNames{"admin"};
	if ($adminurl !~ /^http/) {
	  $adminurl = $closedShop::urltofolder . "/" . $adminurl;
	}

	  # send email to customer
	  closedShop::SendEmail($ue,"Thank your for your order",$body);
	  
	  $body = qq|
An order has been placed by $un - $ue

Please log into the admininstration panel to process it
$adminurl

|;
	  # send email to merchant
	  closedShop::SendEmail($closedShop::CompanyEmail,"An order has been placed",$body);
	
	  my $cookie1 = new CGI::Cookie(-name=>'cart',-value=>"");
	  my $cookie2 = new CGI::Cookie(-name=>'oid',-value=>"");
	  print header(-Cookie=>[$cookie1,$cookie2],-type => "text/html");
	  
		my $finalpage = $closedShop::website;
		if ($closedShop::CompletedOrderURL ne "" && 
		    $closedShop::CompletedOrderURL ne "http://") {
		  $finalpage = $closedShop::CompletedOrderURL
		}
		
	  closedShop::Header("nohttp");
	  print qq|
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	    $closedShop::font <font size="3"><font color="blue">Checkout Process 
		- Complete</font></th></tr>
		<tr bgcolor="$closedShop::mc"><td colspan="2">$closedShop::font
		Thank you for your purchase. You will recieve an email confirmation shortly, and
		another one when the order has been fulfilled and shipped. 
		<br>
		Thank you for shopping with us and we look forward to doing business with you again.<br>
		<br>
		<div align="center"><a href="$finalpage">Return to Homepage</a></div>
		</td></tr></table>
		|;
	}
	elsif ($query->param("selected") == 3) {
	  my $cc = $query->param("cc"); $cc =~ tr/a-zA-Z0-9-_ '-//cd;
	  my $ccnum = $query->param("ccnum"); $ccnum =~ tr/0-9//cd;
	  my $ccname = $query->param("ccname"); $ccname =~ tr/a-zA-Z0-9-_ ,.'-//cd;
	  my $ccmonth = $query->param("ccmonth"); $ccmonth =~ tr/0-9//cd;
	  my $ccyear = $query->param("ccyear"); $ccyear =~ tr/0-9//cd;
	  
	  $ccnum = ValidateCard($ccnum);
	  
	  my %cookies = fetch CGI::Cookie;
      my $cart = $cookies{'cart'}->value if (%cookies); 
	  my $oid = $cookies{'oid'}->value if (%cookies);
	  
	  if (!$ccnum) {
	     closedShop::Header();
		 print "$closedShop::font Your credit card number is not valid";
		 CREDITCARD();
		  return closedShop::Footer();
	  }
	  if (!$cc || !$ccname || !$ccmonth || !$ccyear) {
	     closedShop::Header();
		 print "$closedShop::font You did not fill in one or more required fields";
		 CREDITCARD();
		  return closedShop::Footer();
	  }
	  
	  my ($session,$uid) = closedShop::GetSessionCookie();
	  if (!closedShop::Authenticate($session,"user")) {
	      closedShop::Header();
	      print qq|$closedShop::font<font color="red">You are not
		  logged in properly</font>|;
		  DISPLAY();
		  return closedShop::Footer();
	  }
	
	   my @ary = split(/~/,$cart);
	  
	   my $k = 1;	  
 	   my $grand = 0;
	   my $shiptotal = 0;
	   foreach (@ary) {
	     my ($id,$qty,$attr) = split(/\|/,$_);
	     my $SQL	= qq| select name,price,shipping_price,description,
	              category from $closedShop::sql_tables{"item"} where
				  id="$id" |;
	     my $sth = closedShop::DatabaseQuery($SQL);
	     my ($name,$price,$ship,$desc,$cat) = $sth->fetchrow_array();
	  
	     my $total = sprintf("%2.f",$qty * ($price+$ship));
	     $grand += $total;
		 $shiptotal += $ship;
	   }
	   
	   my $SQL = qq|update $closedShop::sql_tables{"order"} set cc="$cc",
	   cc_name="$ccname",cc_year="$ccyear",cc_month="$ccmonth",cc_num="$ccnum",
	   item_ids="$cart",ship_total="$shiptotal",sub_total="$grand"
	   where id = "$oid" and user_id="$uid"|;
	   my $sth = closedShop::DatabaseQuery($SQL);
	  	
	  closedShop::Header();
	  print qq|<form action="$program_name" method="POST">$closedShop::font
	    <input type="hidden" name="action" value="Checkout">
	    <input type="hidden" name="selected" value="4">
		<input type="hidden" name="oid" value="$oid">
		<b>Please Review your order before continuing</b><br>
		<table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	    $closedShop::font <font size="3"><font color="blue">Checkout Process 
		- Step 4</font></th></tr>

        <tr bgcolor="$closedShop::mc">
        <td colspan="2"><div align="center">
		$closedShop::font<strong>Review Order</strong></div></td></tr>
		
		<tr bgcolor="$closedShop::mc">
        <td >$closedShop::font
		<strong>Billing Info</strong><br>
		|;
		$SQL	= qq| select b_name,b_address,b_city,b_state,
                      b_zip,b_phone,s_name,s_address,s_city,s_state,
                      s_zip,s_phone,item_ids,cc_name,cc,cc_num,cc_month,cc_year
					  from $closedShop::sql_tables{"order"} 
		              where id="$oid" and user_id="$uid" |;
	    $sth = closedShop::DatabaseQuery($SQL);
	    my ($b_name,$b_address,$b_city,$b_state,$b_zip,$b_phone,$s_name,$s_address,
		    $s_city,$s_state,$s_zip,$s_phone,$item_ids,$cc_name,$card,
			$cc_num,$cc_month,$cc_year) = $sth->fetchrow_array();
		my $cc_num2 = $cc_num;
		$cc_num = "**** **** **** " . substr($cc_num,12,4);
		
		print qq|
		$b_name<br>
		$b_address<br>
		$b_city, $b_state $b_zip<br>
		$b_phone
		</td>
		<td >$closedShop::font
		<strong>Shipping Info</strong><br>
		$s_name<br>
		$s_address<br>
		$s_city, $s_state $s_zip<br>
		$s_phone
		</td></tr>
		<tr bgcolor="$closedShop::mc">
        <td colspan="2">$closedShop::font
		<strong>Payment Info</strong><br>
		Credit Card: $closedShop::AcceptedCards{$card}<br>
		Name on Card: $cc_name<br>
		Card Number: $cc_num<br>
		Exp. Date: $cc_month / $cc_year		
		</td></tr>
		<tr bgcolor="$closedShop::mc"><td colspan="2">
		$closedShop::font
		<b>Product Info</b>
		<center>
		<table width="100%" border="1" cellpadding="0" cellspacing="0">
		<tr bgcolor="$closedShop::mc">
		<td>$closedShop::font<strong>Name</strong></td>
		<td>$closedShop::font<strong>Qty</strong></td>
		<td>$closedShop::font<strong>Attributes</strong></td>
		<td>$closedShop::font<strong>Price</strong></td>
		<td>$closedShop::font<strong>Shipping Price</strong></td>
		<td>$closedShop::font<strong>Total Cost</strong></td>
		</tr>
		|;
		
		@ary = split(/~/,$item_ids);
		$k = 1;	  
	    $grand = 0;
		foreach (@ary) {
	  	  my ($id,$qty,$attr) = split(/\|/,$_);
		  my @attr = split(/\?/,$attr); $attr = join("<br>",@attr);
	  	  $SQL	= qq| select name,price,shipping_price,description,
	              category from $closedShop::sql_tables{"item"} where
				  id="$id" |;
	      $sth = closedShop::DatabaseQuery($SQL);
	      my ($name,$price,$ship,$desc,$cat) = $sth->fetchrow_array();
	  
	      my $total = sprintf("%.2f",$qty * ($price+$ship));
	      $grand += $total;

	      print qq|
	   	  <tr bgcolor="$closedShop::mc">
		  <td>$closedShop::font <a 
		  href="$closedShop::FileNames{item}?action=ShowItem&id=$id">$name</a></td>
		  <td>$closedShop::font $qty</td>
		  <td>$closedShop::font $attr</td>
		  <td>$closedShop::font $qty x \$$price</td>
		  <td>$closedShop::font $qty x \$$ship</td>
		  <td>$closedShop::font \$ $total</td>
		  </tr>  |;	
	    }  
	
		my $tax = 0.00;
		if ($closedShop::TaxState eq uc($b_state)) {
		  $tax = sprintf("%.2f",$closedShop::TaxRate * $grand);
		}	
	
		my $total_cost = $tax + $grand;
		$total_cost = sprintf("%.2f",$total_cost);
		$grand = sprintf("%.2f",$grand);
	
	    $SQL = qq|update $closedShop::sql_tables{"order"} set tax="$tax",
	    total="$total_cost" where id = "$oid" and user_id="$uid"|;
	    $sth = closedShop::DatabaseQuery($SQL);

		my $vrate = ($closedShop::TaxRate * 100);
	    print qq|<tr bgcolor="$closedShop::mc">
		<td colspan="4" align="right">&nbsp;</td><td>$closedShop::font
     	<strong>Sub Total: </td><td>$closedShop::font \$ $grand</strong></td></tr>
		<tr bgcolor="$closedShop::mc"><td colspan="4" align="right">&nbsp;</td>
		<td>$closedShop::font
     	<strong>Tax Total (for $closedShop::TaxState at $vrate%):</td><td>$closedShop::font
		 \$ $tax</strong></td></tr>
		<tr bgcolor="$closedShop::mc"><td colspan="4" align="right">&nbsp;</td><td>$closedShop::font
     	<strong>Grand Total:</td><td>$closedShop::font \$ $total_cost</strong></td></tr>
		</table></td></tr>|;
		
		if ($total_cost < $closedShop::MinimumOrder) {
		  print qq|
		<tr bgcolor="$closedShop::mc">
        <td align="" colspan="2">$closedShop::font<div align="center">
		<b>Your order is below the minimum (\$$closedShop::MinimumOrder) amount</b></div>
		</td></tr>	|;
		}
		elsif ($cc_num2 eq "1111111111111111") {
				  print qq|
		<tr bgcolor="$closedShop::mc">
        <td align="" colspan="2">$closedShop::font<div align="center">
		<b>Your credit card number is not real</b></div>
		</td></tr>	|;
		}
		else {
		  print qq|
		<tr bgcolor="$closedShop::mc">
        <td align="" colspan="2">$closedShop::font<div align="center">
		<input type="submit" value="Complete Order"></div>
		</td></tr>	|;
    }	
		
		print qq|	</table>	|;
	}
	elsif ($query->param("selected") == 2) {
	  my $shipname = $query->param("shipname"); 
	  $shipname =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	 
	  my $shipaddress = $query->param("shipaddress"); 
	  $shipaddress =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  
	  my $shipcity = $query->param("shipcity"); 
	  $shipcity =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  
	  my $shipstate = $query->param("shipstate"); 
	  $shipstate =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  
	  my $shipzip = $query->param("shipzip"); 
	  $shipzip =~ tr/0-9-_ -//cd;
	  
	  my $shipphone = $query->param("shipphone");
	  $shipphone =~ tr/0-9-_ ()-//cd;
	 
	  my $billname = $query->param("billname"); 
	  $billname =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  $billname = $shipname if (!$billname);
	  
	  my $billaddress = $query->param("billaddress"); 
	  $billaddress =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	  $billaddress = $shipaddress if (!$billaddress);
	  
	  my $billcity = $query->param("billcity"); 
	  $billcity =~ tr/a-zA-Z0-9-_ .,:;'&-//cd;
	  $billcity = $shipcity if (!$billcity);
	  
	  my $billstate = $query->param("billstate"); 
	  $billstate =~ tr/a-zA-Z0-9-_ .,:-//cd;
	  $billstate = $shipstate if (!$billstate);
	  
	  my $billzip = $query->param("billzip"); 
	  $billzip =~ tr/0-9-_ -//cd;
	  $billzip = $shipzip if (!$billzip);
	  
	  my $billphone = $query->param("billphone"); 
	  $billphone =~ tr/0-9-_ ()-//cd;
	  $billphone = $shipphone if (!$billphone);

	  my ($session,$uid) = closedShop::GetSessionCookie();
	  
	  if (!$shipname || !$shipaddress || !$shipcity || !$shipstate || 
	  !$shipzip  || !$billname || !$billaddress || !$billcity || 
	  !$billstate || !$billzip) {
	     closedShop::Header();
		 print "$closedShop::font You did not fill in one or more required fields";
		 SHIPBILL($uid);
		 return closedShop::Footer();
	  }
	 	  
	  if (!closedShop::Authenticate($session,"user")) {
	  	closedShop::Header();
	    print qq|$closedShop::font<font color="red">You are not
		logged in properly</font>|;
		DISPLAY();
		return closedShop::Footer();
	  }
	 
	  my $SQL = qq|insert into $closedShop::sql_tables{"order"} set user_id="$uid",s_name="$shipname",
	  s_address="$shipaddress",s_city="$shipcity",s_state="$shipstate",
	  s_zip="$shipzip",s_phone="$shipphone",b_name="$billname",
	  b_address="$billaddress",b_city="$billcity",b_state="$billstate",
	  b_zip="$billzip",b_phone="$billphone" |;
	  my $sth = closedShop::DatabaseQuery($SQL);
	  my $order_id = $sth->{'mysql_insertid'};
	  	  
		$SQL = qq|update $closedShop::sql_tables{"user"} set 
	  address="$billaddress",city="$billcity",state="$billstate",
	  zip="$billzip",phone="$billphone" where id="$uid" |;
	  $sth = closedShop::DatabaseQuery($SQL);		
				
	  my $cookie = new CGI::Cookie(-name=>'oid',-value=>$order_id);
	  print header(-Cookie=>[$cookie],
				-expires =>  '+1d',
				-type => "text/html" );
	  closedShop::Header("nohttp");
	  CREDITCARD();
	}
	elsif ($query->param("selected") == 1) {
	  if ($query->param("new")) {
	   
	    my $name = $query->param("name"); $name =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	    my $email = $query->param("email2");  $email =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
		my $password = $query->param("password2"); $password =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
		my $confirm = $query->param("confirm"); $confirm =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
		
		if ($password ne $confirm) {
		  closedShop::Header();
		  print qq|$closedShop::font<font color="red">Your passwords
		  do not match</font>|;
		  DISPLAY();
  		  return closedShop::Footer();
		}
		if (!$name || !$email || !$password) {
		  closedShop::Header();
		  print qq|$closedShop::font<font color="red">Please fill out
		  all fields</font>|;
		  DISPLAY();
  		  return closedShop::Footer();
		}
		my $SQL	= qq| select email from $closedShop::sql_tables{"user"} 
		              where email="$email" |;
	    my $sth = closedShop::DatabaseQuery($SQL);
	    my ($temail) = $sth->fetchrow_array();
		if ($temail) {
		  closedShop::Header();
		  print qq|$closedShop::font<font color="red">The email address
		  has already been taken.</font>|;
		  DISPLAY();
		  return closedShop::Footer();
		}
		$SQL = qq| insert into $closedShop::sql_tables{"user"} 
		           set email="$email",name="$name",password="$password" |;
	    $sth = closedShop::DatabaseQuery($SQL);
		my $new_id = $sth->{'mysql_insertid'};
		LoginUser($new_id);
		closedShop::Header("nohttp");
		SHIPBILL($new_id);
		return closedShop::Footer();
	  }
	  elsif ($query->param("login")) {
	    my $email = $query->param("email"); $email =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
		my $password = $query->param("password1"); $password =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
		my $SQL	= qq| select id from $closedShop::sql_tables{"user"} 
		              where email="$email" and password = "$password" |;
	    my $sth = closedShop::DatabaseQuery($SQL);
	    my ($uid) = $sth->fetchrow_array();
	    if (!$uid || !$email || !$password) {	
		  closedShop::Header();
		  print qq|$closedShop::font<font color="red">There is no matching record
		  for that email address and password</font>|;
		  DISPLAY();
		}
		else {	
		  LoginUser($uid);
		  closedShop::Header("nohttp");
		  SHIPBILL($uid);
	    }
	  }
	}
	else {
	  closedShop::Header();
	  my ($session,$uid) = closedShop::GetSessionCookie();
	  if (closedShop::Authenticate($session,"user")) {
	    SHIPBILL($uid);
	  }
	  else {
	    DISPLAY();
	  }
	}
	closedShop::Footer();  
}

###############################################################################
# FUNCTION:  CREDITCARD
# DESCRIPTION: text for getting CC info
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub CREDITCARD 
{
	print qq|$closedShop::font
    <form action="$program_name" method="POST">
    <input type="hidden" name="action" value="Checkout">
    <input type="hidden" name="selected" value="3">
	<br>Please fill out all fields<br>
	<table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
    $closedShop::font <font size="3"><font color="blue">Checkout Process 
	- Step 3</font></th></tr>
    <tr bgcolor="$closedShop::mc">
    <td colspan="2"><div align="center">
	$closedShop::font<strong>Payment Information</strong></div></td></tr>
   <tr bgcolor="$closedShop::mc"><td>$closedShop::font Credit Card Type: </td>
 	<td>$closedShop::font 
	<select name="cc">|;
	foreach (keys %closedShop::AcceptedCards) {
	  print qq|<option value="$_">$closedShop::AcceptedCards{$_}</option>\n|;
	}
	print qq|
	</select>
	</td></tr>
    <tr bgcolor="$closedShop::mc"><td>$closedShop::font Credit Card Number: </td>
  	<td>$closedShop::font <input type="text" name="ccnum" value="" 
	size="16" maxlength="16"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font Name on Card: </td>
  	<td>$closedShop::font <input type="text" name="ccname" value="" 
	size="20" ></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font Exp. Date: </td>
  	<td>$closedShop::font <input type="text" name="ccmonth" value="" 
	size="2" maxlength="2"> / <input type="text" name="ccyear" value="" 
	size="4" maxlength="4"></td></tr>
	<tr bgcolor="$closedShop::mc"><td colspan="2"><div align="center">$closedShop::font
  	<input type="submit" name="new" value="Review Order -->"></div></td></tr>
	</table>
    |;
}

###############################################################################
# FUNCTION:  SHIPBILL
# DESCRIPTION: text for shipping and billing
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub SHIPBILL
{
	my $uid = shift;
	my $SQL	= qq| select name,address,city,state,zip,phone
		          from $closedShop::sql_tables{"user"} where id = "$uid" |;
	my $sth = closedShop::DatabaseQuery($SQL);
	my ($name,$address,$city,$state,$zip,$phone) = $sth->fetchrow_array();
		  
	print qq|$closedShop::font
	<form action="$program_name" method="POST">
	<input type="hidden" name="action" value="Checkout">
	<input type="hidden" name="selected" value="2">
	<br>Please fill out all fields<br>
	<table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	$closedShop::font <font size="3"><font color="blue">Checkout Process 
	- Step 2</font></th></tr>
     <tr bgcolor="$closedShop::mc">
	 <td colspan="2"><div align="center">$closedShop::font<strong>Shipping 
	Information</strong></div></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font Name </td>
	<td>$closedShop::font <input type="text" name="shipname" value="$name" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font Address </td>
	<td>$closedShop::font <textarea rows="2" cols="30" name="shipaddress">$address</textarea></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font City </td>
	<td>$closedShop::font <input type="text" name="shipcity" value="$city" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font State </td>
	<td>$closedShop::font <input type="text" name="shipstate" value="$state" maxlength="2" size="2"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font Zip </td>
	<td>$closedShop::font <input type="text" name="shipzip" value="$zip" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font Phone </td>
	<td>$closedShop::font <input type="text" name="shipphone" value="$phone" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc">
	<td colspan="2"><div align="center">$closedShop::font<strong>Billing 
	Information</strong><br>Only fill out if different from shipping</div></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font Name </td>
	<td>$closedShop::font <input type="text" name="billname" value="$name" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font Address </td>
	<td>$closedShop::font <textarea rows="2" cols="30" name="billaddress">$address</textarea></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font City </td>
	<td>$closedShop::font <input type="text" name="billcity" value="$city" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font State </td>
	<td>$closedShop::font <input type="text" name="billstate" value="$state" maxlength="2" size="2"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font Zip </td>
	<td>$closedShop::font <input type="text" name="billzip" value="$zip" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font Phone </td>
	<td>$closedShop::font <input type="text" name="billphone" value="$phone" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td colspan="2"><div align="center">$closedShop::font
	<input type="submit" name="new" value="Payment Info -->"></div></td></tr>
	</table>
	|;
}

sub DISPLAY
{
	print qq|$closedShop::font
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="Checkout">
	  <input type="hidden" name="selected" value="1">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	  $closedShop::font <font size="3"><font color="blue">Checkout Process - Step 1</font></th></tr>

	  <tr bgcolor="$closedShop::mc">
	  <td colspan="2"><div align="center">$closedShop::font<strong>Returning Shoppers</strong><br>
	  Please enter your email address and password</div></td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Email Address: </td>
	  <td>$closedShop::font <input type="text" name="email" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Password: </td>
	  <td>$closedShop::font <input type="password" name="password1" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc">
	  <td colspan="2"><div align="center">$closedShop::font
	  <input type="submit" name="login" value="Login"></div></td></tr>

	  <tr bgcolor="$closedShop::mc"><td colspan="2">&nbsp;</td></tr>
	  
	  <tr bgcolor="$closedShop::mc">
	  <td colspan="2"><div align="center">$closedShop::font<strong>New Shoppers</strong><br>
	  Please fill required fields</div></td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Name: </td>
	  <td>$closedShop::font <input type="name" name="name" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Email Address: </td>
	  <td>$closedShop::font <input type="text" name="email2" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Password: </td>
	  <td>$closedShop::font <input type="password" name="password2" size="20"></td></tr>
  	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Confirm Password: </td>
	  <td>$closedShop::font <input type="password" name="confirm" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td colspan="2"><div align="center">$closedShop::font
	  <input type="submit" name="new" value="Create Account"></div></td></tr>
	  </table>|;
}


###############################################################################
# FUNCTION:  ViewCart
# DESCRIPTION: view the shoppers cart
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ViewCart
{
	my $message = shift;
	closedShop::Header("$message");
  
	print qq|
	<table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="7">
	$closedShop::font <font size="3"><font color="blue">Shopping Cart</font></th></tr>
	<tr bgcolor="$closedShop::mc">
	<td>$closedShop::font<strong>Name</strong></td>
	<td>$closedShop::font<strong>Qty</strong></td>
	<td>$closedShop::font<strong>Attributes</strong></td>
	<td>$closedShop::font<strong>Price</strong></td>
	<td>$closedShop::font<strong>Shipping Price</strong></td>
	<td>$closedShop::font<strong>Total Cost</strong></td>
	<td>$closedShop::font<strong>Remove</strong></td>
	</tr>|;

    my %cookies = fetch CGI::Cookie;
    my $cart = ""; my @ary = ();
    $cart = $cookies{'cart'};
	if ($cart) {
	  $cart = $cookies{'cart'}->value;
	  @ary = split(/~/,$cart) if (defined($cart));
	}
  
	my $k = 1;	  
	my $grand = 0;
	foreach (@ary) {
	  my ($id,$qty,$attr) = split(/\|/,$_);
	  my @attr = split(/\?/,$attr); $attr = join("<br>",@attr);
	  my $SQL	= qq| select name,price,shipping_price,description,
	              category from $closedShop::sql_tables{"item"} where
				  id="$id" |;
	  my $sth = closedShop::DatabaseQuery($SQL);
	  my ($name,$price,$ship,$desc,$cat) = $sth->fetchrow_array();
	  
	  my $total = sprintf("%.2f",$qty * ($price+$ship));
	  $grand += $total;
	  
	  print qq|
	   	  <tr bgcolor="$closedShop::mc">
		  <td>$closedShop::font <a 
		  href="$closedShop::FileNames{item}?action=ShowItem&id=$id">$name</a></td>
		  <td>$closedShop::font $qty</td>
		  <td>$closedShop::font $attr &nbsp;</td>
		  <td>$closedShop::font $qty x \$$price</td>
		  <td>$closedShop::font $qty x \$$ship</td>
		  <td>$closedShop::font \$$total</td>
		  <td>$closedShop::font 
		  <a href="$program_name?action=DeleteProduct&id=$id">Delete Item</a></td>
		  </tr>  |;	
	  $k++;
	} 
	
	if ($k == 1) {
	print qq|<tr bgcolor="$closedShop::mc"><td colspan="7">$closedShop::font
	<center><strong><br>
   There are no items in your shopping cart</strong><br><br></td></tr>
	|;
	}

	
	my $lastitem = ""; 
	
	my $contshop = qq|$closedShop::FileNames{item}|;
	if ($closedShop::ContinueShoppingURL ne "" && 
	    $closedShop::ContinueShoppingURL ne "http://") {
    $contshop = $closedShop::ContinueShoppingURL;
	}
	
	$grand = sprintf("%.2f",$grand);
	print qq|<tr bgcolor="$closedShop::mc"><td colspan="5" align="right">&nbsp;</td>
	<td>$closedShop::font
	<strong>Sub Total:</td><td>$closedShop::font \$$grand</strong></td></tr>
	<tr bgcolor="$closedShop::mc"><td colspan="3">$closedShop::font 
	<a href="$contshop"><b>Continue Shopping</b></a></td>
	<td>$closedShop::font <a href="$program_name?action=Checkout"><b>Checkout</b></a>
	</td>
	<td colspan="4">|;
	
	if ($grand < $closedShop::MinimumOrder) {
	  print qq|$closedShop::font
		The minimum amount 
    for an order is \$$closedShop::MinimumOrder|;
	}
	else {
	  print qq|&nbsp;|;
	}
	print qq|
	</td>
	</tr></table>
	<br>
	|;
	closedShop::Footer();

}

###############################################################################
# FUNCTION:  DeleteProduct
# DESCRIPTION: delete a product
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub DeleteProduct
{
	my $id = $query->param("id"); $id =~ tr/0-9//cd;

    my %cookies = fetch CGI::Cookie;
    my $cart = $cookies{'cart'}->value if (%cookies);

	my $ncart = "";
	my @ary = split(/~/,$cart);
	foreach (@ary) {
	  my ($i,$q,$a) = split(/\|/,$_);
	  if ($i != $id) {
	    $ncart .= qq@$i|$q|$a~@;
	  }
	}
	
	my $cookie = new CGI::Cookie(-name=>'cart',-value=>"$ncart");

	print header(-Cookie=>[$cookie],
				 -type => "text/html",
				 );
	print qq|<html><head><meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
	<meta http-equiv="refresh" content="0; URL=$program_name">
    <title></title></head><body>&nbsp;</body></html> |;
}

################################################################################
# FUNCTION:  LoginUser
# DESCRIPTION: process screen
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub LoginUser
{
      my $uid = shift;
	  my $session = closedShop::CreateSession();
	  my $SQL = qq| update $closedShop::sql_tables{"user"} set session="$session" where id = "$uid" |;
	  my $sth = closedShop::DatabaseQuery($SQL);
	  closedShop::SetSessionCookie($session,$uid);
}

################################################################################
# FUNCTION:  ValidateCard
# DESCRIPTION: validates card - courtesy of Business::CreditCard
# from CPAN - thank you Ivan Kohler 
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub ValidateCard 
{
    my ($number) = @_;
	my $return_number = $number;
    my ($i, $sum, $weight);
    
    return 0 if $number =~ /[^\d\s]/;

    $number =~ s/\D//g;

    return 0 unless length($number) >= 13 && 0+$number;

    for ($i = 0; $i < length($number) - 1; $i++) {
        $weight = substr($number, -1 * ($i + 2), 1) * (2 - ($i % 2));
        $sum += (($weight < 10) ? $weight : ($weight - 9));
    }

    return $return_number if substr($number, -1) == (10 - $sum % 10) % 10;
	# added for testing purposes
	return $return_number if ($return_number == "1111111111111111");
    return 0;
}







