#!/usr/bin/perl 

################################################################################
# 
# PROGRAM:  Install.pl - from closedShop - Open Shopping Cart 
#
# RCSID:  $Id$
#
# DESCRIPTION: Install the program
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:  
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-06-12   cbf    Created
# 2001-06-22   cbf    Added support for renaming file names
# 2001-07-03   cbf    Added a list of states to choose from
# 2001-07-24   cbf    Mailprogram can be customized
# 2001-08-12   cbf    All input validated
# 2001-09-08   cbf    Added Secure Image URL
# 2001-09-27   dsb    Added a merchant username
# 2001-10-18   cbf    Added database location
# 2002-05-12   cbf    Added minimum order amount, added continue shopping vars,
#                       final return vars
################################################################################

use strict;
use CGI;
use DBI;

###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query = new CGI;
my $program_name = "Install.pl";

my $varsfile = "closedShop.vars";
	   
my $font = qq| <font face="arial, verdana" size="3"> |;
my $tc = "#99cc99"; my $mc = "#66ffff";
################################################################################

Main();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main
{
	my $action = $query->param('action');
	if (IsInstalled()) { 
	  print "Location: Admin.pl\n\n";
	}
	elsif ($action eq "Install") {
	  Install();
	}
	else {
	  InstallScreen("");
	}
}

################################################################################
# FUNCTION:  IsInstalled
# DESCRIPTION: checks to see if it has been run once or not
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub IsInstalled
{
	if (-e "$varsfile")  { return 1; }
	else                 { return 0; }
}

################################################################################
# FUNCTION:  Install
# DESCRIPTION: install 
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub Install
{
	use closedShop;
	open(FILE,">$varsfile");
	
	if (!(-e $varsfile)) { 
	  return closedShop::ErrorMessage("Can't create the
	  closedShop.vars file needed to store variables. Please ensure that
	  this program can create files in this folder or else this installation
	  will be troublesome");
	}
	foreach ($query->param()) {
	  if ($_ eq "AdminUsername" || $_ eq "AdminPassword" 
	  || $_ eq "MerchantUsername" || $_ eq "MerchantPassword") { next; }
	  my @var = $query->param("$_"); 
	  my $var = join(",",@var); $var =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*[]{}\/()?-//cd;
	  print FILE qq|$_~$var\n|; 
	}
	close(FILE);
	
	my $AUser = $query->param("AdminUsername"); $AUser =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	my $APass = $query->param("AdminPassword"); $APass =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	my $MUser = $query->param("MerchantUsername"); $MUser =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	my $MPass = $query->param("MerchantPassword"); $MPass =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	
	
	closedShop::Initialize();
	
	closedShop::CreateSQL();
	closedShop::CreateAdmin($AUser,$APass);
	closedShop::CreateAdmin($MUser,$MPass);
	
	Header();
	print qq|You have successfully (fingers crossed) installed closedShop.
	If you have trouble, and believe closedShop did not install properly,
	the best bet is to delete the file called closedShop.vars in this directory
	and run Install.pl again.
	<br>
	<br>
	Your next step might be to login into the <a href="$closedShop::urltofolder/$closedShop::FileNames{admin}">Admin Panel</a>
	
	
	|;
	Footer();
}


################################################################################
# FUNCTION:  InstallScreen
# DESCRIPTION: install screen
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub InstallScreen
{
	my $message = shift;
	Header();
	
	print qq|<br>
	<b>Please fill out all variables, otherwise closedShop may
	not work properly. <br>
	Feel free to change any of the predefined variables. <br>
	They are there only as suggestions.</b><br>
	<br>
	
	<table width="400" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - Company Info</b></font></td></tr>
	<form action="$program_name" method="POST" enctype="multipart/form-data">
	<input type="hidden" name="action" value="Install">
	
	<tr bgcolor="$mc"><td>$font<b>Company Name</b>: </font></td><td>$font 
	<input type="text" name="CompanyName" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Company Email</b>: </font></td><td>$font 
	<input type="text" name="CompanyEmail" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Company Phone</b>: </font></td><td>$font
	<input type="text" name="CompanyPhone" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Company Address</b>: </font></td><td>$font 
	<input type="text" name="CompanyAddress" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Company City</b>: </font></td><td>$font 
	<input type="text" name="CompanyCity" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Company State</b>: </font></td><td>$font 
	<select name="CompanyState">
	<OPTION  VALUE="AL">Alabama
	<OPTION  VALUE="AK">Alaska
	<OPTION  VALUE="AZ">Arizona
	<OPTION  VALUE="AR">Arkansas
	<OPTION  VALUE="CA">California
	<OPTION  VALUE="CO">Colorado
	<OPTION  VALUE="CT">Connecticut
	<OPTION  VALUE="DC">District Columbia
	<OPTION  VALUE="DE">Delaware
	<OPTION  VALUE="FL">Florida
	<OPTION  VALUE="GA">Georgia
	<OPTION  VALUE="HI">Hawaii
	<OPTION  VALUE="ID">Idaho
	<OPTION  VALUE="IL">Illinois
	<OPTION  VALUE="IN">Indiana
	<OPTION  VALUE="IA">Iowa
	<OPTION  VALUE="KS">Kansas
	<OPTION  VALUE="KY">Kentucky
	<OPTION  VALUE="LA">Louisiana	
	<OPTION  VALUE="ME">Maine
	<OPTION  VALUE="MD">Maryland
	<OPTION  VALUE="MA">Massachusetts
	<OPTION  VALUE="MI">Michigan
	<OPTION  VALUE="MN">Minnesota
	<OPTION  VALUE="MS">Mississippi
	<OPTION  VALUE="MO">Missouri
	<OPTION  VALUE="MT">Montana
	<OPTION  VALUE="NE">Nebraska
	<OPTION  VALUE="NV">Nevada
	<OPTION  VALUE="NH">New Hampshire
	<OPTION  VALUE="NJ">New Jersey
	<OPTION  VALUE="NM">New Mexico
	<OPTION  VALUE="NY">New York
	<OPTION  VALUE="NC">North Carolina
	<OPTION  VALUE="ND">North Dakota
	<OPTION  VALUE="OH">Ohio
	<OPTION  VALUE="OK">Oklahoma
	<OPTION  VALUE="OR">Oregon
	<OPTION  VALUE="PA">Pennsylvania
	<OPTION  VALUE="PR">Puerto Rico
	<OPTION  VALUE="RI">Rhode Island
	<OPTION  VALUE="SC">South Carolina
	<OPTION  VALUE="SD">South Dakota
	<OPTION  VALUE="TN">Tennessee
	<OPTION  VALUE="TX">Texas
	<OPTION  VALUE="UT">Utah
	<OPTION  VALUE="VT">Vermont
	<OPTION  VALUE="VA">Virginia
	<OPTION  VALUE="WA">Washington
	<OPTION  VALUE="WV">West Virginia
	<OPTION  VALUE="WI">Wisconsin
	<OPTION  VALUE="WY">Wyoming
	</select>
	</td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Company Zip</b>: </font></td><td>$font 
	<input type="text" name="CompanyZip" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Tax Rate for State</b>: </font></td><td>$font 
	<input type="text" name="TaxRate" value="" size="5"></td></tr>
	 
	 	<tr bgcolor="$mc"><td>$font<b>Minumum Amount for order:</b><br>
   (leave blank for no min.) </font></td><td>$font \$ 
	<input type="text" name="MinimumOrder" value="" size="5"></td></tr>
	 
	 <tr bgcolor="$mc"><td>$font<b>Select Accepted Credit Cards</b>:</td><td>$font 
	  
	<input type="checkbox" name="creditcard" value="mastercard">Mastercard<br>
	<input type="checkbox" name="creditcard" value="visa">Visa<br>
	<input type="checkbox" name="creditcard" value="discover">Discover<br>
	<input type="checkbox" name="creditcard" value="amex">American Express<br>
	<input type="checkbox" name="creditcard" value="diner">Diner's Club<br>
	
	 </td></tr>
	 
	 <tr bgcolor="$mc"><td>$font<b>Create Categories</b>:<br>
	 <i>Create as many as you need</i></td><td>$font 
	  
	Category 1: <input type="text" name="category1" value="" size="20"><br>
	Category 2: <input type="text" name="category2" value="" size="20"><br>
	Category 3: <input type="text" name="category3" value="" size="20"><br>
	Category 4: <input type="text" name="category4" value="" size="20"><br>
	Category 5: <input type="text" name="category5" value="" size="20"><br>
	Category 6: <input type="text" name="category6" value="" size="20"><br>
	Category 7: <input type="text" name="category7" value="" size="20"><br>
	Category 8: <input type="text" name="category8" value="" size="20"><br>
	Category 9: <input type="text" name="category9" value="" size="20"><br>
	Category 10: <input type="text" name="category10" value="" size="20"><br>
	Category 11: <input type="text" name="category11" value="" size="20"><br>
	Category 12: <input type="text" name="category12" value="" size="20"><br>
	
	 </td></tr>
	 
	 </table><br><br>
	 
	  <table width="400" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - Administration Details</b></font></td></tr>
	
	<tr><td colspan="2" bgcolor="$mc">
	$font Two administrative users will be created. The Admin User can access and edit all
	program variables being created on this screen. The Merchant User will not be able to
	access or edit the server specific variables.
	</font></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Admin Username</b>: </font></td><td>$font 
	<input type="text" name="AdminUsername" value="admin" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Admin Password</b>: </font></td><td>$font 
	<input type="text" name="AdminPassword" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Merchant Username</b>: </font></td><td>$font 
	<input type="text" name="MerchantUsername" value="merchant" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Merchant Password</b>: </font></td><td>$font 
	<input type="text" name="MerchantPassword" value="" size="20"></td></tr>

	
	</table><br>
	<br>
	
	 
	 <table width="400" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - SQL Database Info</b></font></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Database Name</b>: </font></td><td>$font 
	<input type="text" name="DatabaseName" value="" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Database Username</b>: </font></td><td>$font 
	<input type="text" name="DatabaseUsername" value="" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Database Password</b>: </font></td><td>$font 
	<input type="text" name="DatabasePassword" value="" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Database Location</b><br>
	<i>Only change if you know what you are doing</i>: </font></td><td>$font 
	<input type="text" name="DatabaseLocation" value="localhost" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Database Table Names</b>: </font></td><td>$font 
	Admin Table: <input type="text" name="admintable" value="closedshop_admin" size="20"><br>
	Item Table: <input type="text" name="itemtable" value="closedshop_item" size="20"><br>
	Order Table: <input type="text" name="ordertable" value="closedshop_order" size="20"><br>
	User Table: <input type="text" name="usertable" value="closedshop_user" size="20"><br>
	
	</td></tr>
		 </table><br><br>
	
	 <table width="400" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - Server Specifics</b></font></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Website URL</b>: <br>
	No Trailing Slash</font></td><td>$font 
	<input type="text" name="website" value="http://" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>URL to closedShop Folder</b>: <br>
	No Trailing Slash</font></td><td>$font 
	<input type="text" name="urltofolder" value="http://" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>URL to Continue Shopping Link</b>:<br>
  (If blank, will return to search page) <br>
	No Trailing Slash</font></td><td>$font 
	<input type="text" name="ContinueShoppingURL" value="http://" size="30"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>URL to Final link after completed order</b>:<br>
  (If blank, will return to homepage of website) <br>
	No Trailing Slash</font></td><td>$font 
	<input type="text" name="CompletedOrderURL" value="http://" size="30"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Server Path to Image Folder</b>: <br>
	No Trailing Slash</font></td><td>$font 
	<input type="text" name="IMAGEPATH" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>URL to Image Folder</b>: <br>
	No Trailing Slash</font></td><td>$font 
	<input type="text" name="IMAGEURL" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>URL to <b>SECURE</b> Image Folder</b>: <br>
	No Trailing Slash</font></td><td>$font 
	<input type="text" name="SecureImageURL" value="" size="30"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Path to Header File</b>: </font></td><td>$font 
	<input type="text" name="header" value="" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Path to Footer File</b>: </font></td><td>$font 
	<input type="text" name="footer" value="" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Mail Program</b>: </font></td><td>$font 
	<input type="text" name="mailprog" value="/usr/bin/sendmail" size="30"></td></tr>
	
	
	</table><br><br>
	
	
	 <table width="400" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - Extras</b></font></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Table Top Colors</b>: </font></td><td>$font 
	<input type="text" name="tc" value="#99CC99" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>Table Main Colors</b>: </font></td><td>$font 
	<input type="text" name="mc" value="##EEEEEE" size="20"></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>Font</b>: </font></td><td>$font 
	<input type="text" name="font" value="<font face='verdana,arial' size='2'>" size="30"></td></tr>

	</table><br><br>
	
		 <table width="400" border="0"><tr><td colspan="2" bgcolor="$tc">
	$font <font color="blue"><b>Installation - File names</b></font></td></tr>
	
		<tr><td colspan="2" bgcolor="$tc">$font<b><i>Important - these must be a full URL if they will be called using cgi-wrap or on a separate secure server. See the README file.</i></b></font></td></tr>
	
	<tr bgcolor="$mc"><td>$font<b>URL for Admin.pl</b>: </font></td><td>$font 
	<input type="text" name="FileName_admin" value="Admin.pl" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>URL for Cart.pl</b>: </font></td><td>$font 
	<input type="text" name="FileName_cart" value="Cart.pl" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>URL for User.pl</b>: </font></td><td>$font 
	<input type="text" name="FileName_user" value="User.pl" size="20"></td></tr>
	<tr bgcolor="$mc"><td>$font<b>URL for Item.pl</b>: </font></td><td>$font 
	<input type="text" name="FileName_item" value="Item.pl" size="20"></td></tr>


	
	</table><br><br>
	
	 <table width="400" border="0"><tr><td colspan="2" bgcolor="$tc"><center>
	$font <font color="blue"><b><input type="submit" value="Install"></b></font></td></tr>
	</table></form>
	 
	</center></div>|;
	
	Footer();
}

################################################################################
# FUNCTION:  Header
# DESCRIPTION: header
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub Header
{ 
	print "Content-type: text/html \n\n";
	print qq|<html><head><title>closedShop Installation</title></head>
	<body bgcolor="white">
	$font
	<h3>closedShop Installation</h3>
	|;
}

################################################################################
# FUNCTION:  Footer
# DESCRIPTION: footer
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub Footer
{
	print qq|
	</body>
	</html>
	|;
}



