#!/usr/bin/perl 

################################################################################
# 
# PROGRAM:  Item.pl - from closedShop - Open Shopping Cart
#
# RCSID:  $Id$
#
# DESCRIPTION: Shows items, allows for searches
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:  
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-05-08   cbf    Created
# 2001-06-07   cbf    Allowed for searching in any category, show attributes
# 2001-06-24   cbf    cosmetic changes
# 2001-08-12   cbf    All input validated
# 2001-09-08   cbf    Added SecureImageURL variable use
# 2001-09-18   cbf    A mark appears if a picture is available
# 2002-01-20   cbf    Allowed for thumbnail displayal of categories
# 2002-02-01   cbf    Only shows intro text for cat if its first page
# 2002-02-11   cbf    Make use of public/private categories
################################################################################

#use CGI::Carp(fatalsToBrowser);
use closedShop;
use strict;
use CGI;
use DBI;
use Fcntl ':flock';

closedShop::Initialize();
###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query = new CGI;
my $program_name = $closedShop::FileNames{"item"};
	
my %actions = (ShowItem => "Show an Item",
	           Search => "Item Search",
			   ShowCategory => "Display a Category"
			   );

###############################################################################

Main();
closedShop::Cleanup();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main
{
	my $action = $query->param('action'); 
	$action =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	$action = "cs" if (!defined($action));
	if (!defined($actions{$action})) { 
	  Search("");
	}
	else {
	  eval ($action);
	}
}

###############################################################################
# FUNCTION:  ShowItem
# DESCRIPTION: displays an item
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ShowItem
{
	my $id = $query->param("id"); $id =~ tr/0-9//cd;
	if (!$id || $id == 0) { $id = 1; }
	
	my $SQL	= qq| select name,price,shipping_price,description,attributes,
	              category from $closedShop::sql_tables{"item"} where
				  id="$id" |;
	my $sth = closedShop::DatabaseQuery($SQL);
	my ($name,$price,$ship,$desc,$attr,$cat) = $sth->fetchrow_array();
	 
	 my @attr = split(/\|/,$attr);
	 my @attr1 = split(/~/,$attr[0]);
	 my @attr2 = split(/~/,$attr[1]);
	 my @attr3 = split(/~/,$attr[2]);
	 
	 my $n1 = shift(@attr1);
	 my $n2 = shift(@attr2);
	 my $n3 = shift(@attr3);
	 
	 my $attribute1 = qq|$n1: <select name="attribute1">|;
	 foreach (@attr1) {
	   $attribute1 .= qq|<option value="$_">$_</option>\n|;
	 }
	 $attribute1 .= qq|</select>|;
	 my $attribute2 = qq|$n2: <select name="attribute2">|;
	 foreach (@attr2) {
	   $attribute2 .= qq|<option value="$_">$_</option>\n|;
	 }
	 $attribute2 .= qq|</select>|;
	 my $attribute3 = qq|$n3: <select name="attribute3">|;
	 foreach (@attr3) {
	   $attribute3 .= qq|<option value="$_">$_</option>\n|;
	 }
	 $attribute3 .= qq|</select>|;
	 
	 $attribute1 = "" if (!$n1);
	 $attribute2 = "" if (!$n2);
	 $attribute3 = "" if (!$n3);
	 
	 $attribute1 = "NOATT" if (!$attribute1 && !$attribute2 && !$attribute3);
	 
	  closedShop::Header();
	  print qq|
	  <form action="$closedShop::FileNames{cart}" method="POST">
	  <input type="hidden" name="action" value="AddProduct">
	  <input type="hidden" name="id" value="$id">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	  $closedShop::font <font size="3"><font color="blue">$name</font></th></tr>
 	  <tr bgcolor="$closedShop::mc"><td>|;

	  if (-e "$closedShop::IMAGEPATH/$id.jpg") {
	    if ($closedShop::FileNames{"item"} =~ /https/) {
		  print qq|<a href="$closedShop::IMAGEURL/$id.jpg"><img border="0" 
		  alt="Click to view larger image" width="220"
		  src="$closedShop::SecureImageURL/$id.jpg"></a><br>\n|;
		}
	    else {
	      print qq|<a href="$closedShop::IMAGEURL/$id.jpg"><img  
		  width="220" border="0" alt="Click to view larger image"
		  src="$closedShop::IMAGEURL/$id.jpg"></a><br>\n|;
		}
	  }
	  else {
	    print qq|$closedShop::font<b><div align="center">No image available at this time</div></b><br>\n|;
	  }
	  
	  print qq|</td><td valign="top"><table border="0" width="100%">
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font <strong>Product Name</strong>:</td>
	  <td>$closedShop::font $name</td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font <strong>Category</strong>:</td>
	  <td>$closedShop::font $closedShop::categories{$cat}</td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font <strong>Price</strong>:</td>
	  <td>$closedShop::font \$$price</td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font <strong>Shipping Cost</strong>:</td>
	  <td>$closedShop::font \$$ship</td></tr>|;
	  
	  if ($attribute1 ne "NOATT") {
	  print qq|
	    <tr bgcolor="$closedShop::mc"><td>$closedShop::font <strong>Attributes</strong>:</td>
	    <td>$closedShop::font $attribute1<br>$attribute2<br>$attribute3 &nbsp;</td></tr>
	    |;
	  }
	  
	  print qq|
	  </table></td></tr>
	  <tr bgcolor="$closedShop::mc"><td colspan="2">$closedShop::font $desc</td></tr>
	  <tr bgcolor="$closedShop::mc"><td colspan="1">
	  $closedShop::font 
	  Quantity: <input type="text" name="quantity" value="1" size="3" maxlength="3">
	  &nbsp;&nbsp;&nbsp;&nbsp;
	  <input type="submit" value="Add Item to Cart"></td></form>
	  <form action="$closedShop::FileNames{cart}" method="POST"><td>  $closedShop::font
	  <input type="hidden" name="action" value="AddWishList">
	  <input type="hidden" name="id" value="$id">
	  <input type="submit" value="Add Item to WishList"></td></tr>
	  </form></table>
	  |;
	  closedShop::Footer();
}


###############################################################################
# FUNCTION:  ShowCategory
# DESCRIPTION: displays an item
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub ShowCategory
{
	my $cat = $query->param("category"); $cat =~ tr/a-zA-Z0-9//cd;
	my $lastitem = $query->param("lastitem"); $lastitem =~ tr/0-9//cd;
	my $startitem = $query->param("startitem"); $startitem =~ tr/0-9//cd;
	my $grouping = $query->param("grouping"); $grouping =~ tr/0-9//cd;
	
	my $SQL	= qq| select id,name,price from $closedShop::sql_tables{"item"} 
	              where category="$cat" order by name |;
	my $sth = closedShop::DatabaseQuery($SQL);
	
	my $SQL2	= qq| select COUNT(id) from $closedShop::sql_tables{"item"} 
	              where category="$cat"  |;
	my $sth2 = closedShop::DatabaseQuery($SQL2);
	my ($numitems) = $sth2->fetchrow_array();
	
	my $mark = qq|<div align="left">(P) = Picture Available</font></div>|;
	if (-e "$closedShop::IMAGEPATH/photoexists.gif") {
	  $mark = qq|<div align="left"><img 
	  src="$closedShop::SecureImageURL/photoexists.gif"> 
	  <font color="Red"> = Photo Available</font></div>|;
	}
	
	my ($intro,$style,$numimages,$status) = closedShop::GetCategoryInfo($cat);
	

	
	$intro .= qq|<BR><BR>|;
	if ($grouping > 1) { $intro = ""; }
	
    closedShop::Header();
	
	if ($status eq "PRIVATE" || $status eq "NOCAT") { 
	  print qq|<br>
	  <table width="300" border="1"><tr bgcolor="$closedShop::tc"><th colspan="1">
	  $closedShop::font <font size="3"><font color="blue">Category not accessible</font></th></tr>
	  <tr bgcolor="$closedShop::mc">
	  <td>$closedShop::font The requested category is not available at the requested time</td>
	  </tr></table><br>
	  <br>  |;
	  return closedShop::Footer();
	}
	
	print qq|$closedShop::font  $intro |;
	 
	 
	  
	if ($style eq "text") {
	  print qq|<center>Jump to: |;
	  foreach (qw/A B C D E F G H I J K L M N O P Q R S T U V W X Y Z/) {
	    print qq|<A href="#$_">$_</a> |;
	  }
	  print qq|</center><br>
	  $mark
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="3">
	  $closedShop::font <font size="3"><font color="blue">Category - 
	  $closedShop::categories{$cat}</font></th></tr>
	  <tr bgcolor="$closedShop::mc">
	  <td>$closedShop::font<strong>Name</strong></td>
	  <td>$closedShop::font<strong>Price</strong></td>
	  <td>$closedShop::font<strong>Cart</strong></td>
	  </tr>|;

	  my $k = 0;
	  my $letter = "";
	  while (my ($id,$name,$price) = $sth->fetchrow_array()) {
	    my $picturemark = "";
		my $firstletter = (split(//,$name))[0];
	    if (-e "$closedShop::IMAGEPATH/$id.jpg") {
	      if (-e "$closedShop::IMAGEPATH/photoexists.gif") {
		    if ($closedShop::FileNames{"item"} =~ /https/) {
	          $picturemark = qq|<img src="$closedShop::SecureImageURL/photoexists.gif"> |;
		    }
		    else {
		      $picturemark = qq|<img src="$closedShop::IMAGEURL/photoexists.gif"> |;
		    }
		  }
		  else {
		    $picturemark = qq|<font color="red">(P)</font> |;
		  }
	    }
      
	  my $printletter = "";
	  if ($firstletter ne $letter) { 
	    $letter = $firstletter; 
		$printletter = $letter; 
	  }
	  print qq|
	   	  <tr bgcolor="$closedShop::mc"><td>
		  $closedShop::font <a name="$printletter">$picturemark</a> <a 
		  href="$program_name?action=ShowItem&id=$id">$name</a></td>
		  <td>$closedShop::font \$$price</td>
		  <td>$closedShop::font <a href="$closedShop::FileNames{cart}?action=AddProduct&id=$id&quantity=1">Add to Cart</a></td>
		  </tr>  |;	
	  $k++;
	  }  
	
	  if ($k == 0) {
	    print qq|
	   	  <tr bgcolor="$closedShop::mc"><td colspan="3">
		  $closedShop::font <b><div align="center">There are currently no items in this category</div>
		  </b></td></tr>|;	
  	  }
	
	 print qq|</table>|;
    }
	elsif ($style eq "thumbnail") {
	  if (!$grouping || $grouping > $numitems)  { $grouping = 1; }

	  my $ng = $grouping + $numimages; 
	  my $og = $grouping - $numimages;
	  if ($og < 1) { $og = 1; }

	  my $ngdisplay = $ng - 1;
	  if ($ngdisplay > $numitems) { $ngdisplay = $numitems; }
	  my %imagesizes = ("1"=>"400","2"=>"250","4"=>"200","8"=>"200","16"=>"100");
	  my %newrow = ("1"=>"1","2"=>"2","4"=>"2","8"=>"2","16"=>"4");
	  print qq|
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc">
	  <th colspan="$newrow{$numimages}">
	  $closedShop::font <font size="3"><font color="blue">Category - 
	  $closedShop::categories{$cat}<br></font><Font size="2">
	  Displaying Items $grouping - $ngdisplay out of $numitems</font></th></tr>|;

	  my $k = 1; my $j = 1; my $t = 1;
	  
	  my $tmpimages = $numimages;
	  if (($numitems - $grouping + 1) < $numimages) {
	    $tmpimages = $numitems - $grouping + 1;
	  }
	  
	  while (my ($id,$name,$price) = $sth->fetchrow_array()) {
	    if ($t < $grouping) { $t++; next; }
    
		if ($k == 1) {
		  print qq|<tr bgcolor="$closedShop::mc">|;
		}

		PrintItem($name,$price,$id,$imagesizes{$numimages});
		
		if ($k == $newrow{$numimages}) {  
		  print qq| </tr> |;	
		  $k = 0;
		}
		if ($tmpimages == $j) {
		  print qq|<tr bgcolor="$closedShop::mc">
		  <td colspan="$newrow{$numimages}"><center>$closedShop::font <A 
		  href="$program_name?action=ShowCategory&category=$cat&grouping=$og">&lt;&lt;
			Previous Page</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<A 
		  href="$program_name?action=ShowCategory&category=$cat&grouping=$ng">Next 
		  Page &gt;&gt;</a>
		  </td></tr>
		  |;
		  last;
		}
	    $k++; $j++;
	  }
	   print qq|</table>|;
	}	 
	 
	closedShop::Footer();
}

###############################################################################
# FUNCTION:  PrintItem
# DESCRIPTION: print item stuff
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub PrintItem
{
	my ($name,$price,$id,$imsize) = @_;
	print qq|<td><centeR>$closedShop::font <A 
		  href="$program_name?action=ShowItem&id=$id">|;

		if ($closedShop::FileNames{"item"} =~ /https/) {
		  if (-e "$closedShop::IMAGEPATH/$id.jpg") {
		    print qq|<img border="0" 
		    alt="Click to view product info" width="$imsize"
		    src="$closedShop::SecureImageURL/$id.jpg">|;
	      }
		  else {
		    print qq|<table border="1"><tr><td>$closedShop::font
			<b><div align="center">No image currently available</div>
			</b></td></tr></table>|;
		  }
		}
	    else {
		  if (-e "$closedShop::IMAGEPATH/$id.jpg") {
	        print qq|<img width="$imsize" border="0" alt="Click to 
		    view product info" src="$closedShop::IMAGEURL/$id.jpg">|;
	      }
		  else {
		    print qq|<table border="1"><tr><td>$closedShop::font
			<b><div align="center">No image currently available</div></b>
			</td></tr></table>|;
		  }
		}

		 print qq|</a><br><A href="$program_name?action=ShowItem&id=$id">$name</a>
		 <br>Price: \$$price - <a 
		 href="$closedShop::FileNames{cart}?action=AddProduct&id=$id&quantity=1">Add 
		 to Cart</a></td>|;

}

###############################################################################
# FUNCTION:  Search
# DESCRIPTION: allows for searching
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Search
{
    closedShop::Header();
	if ($query->param("selected") == 1) {
	  my $SQL = qq|select id,name,price,category from 
	               $closedShop::sql_tables{"item"} where 1=1 |;
	  if ($query->param("name")) {
	    my $name = $query->param("name"); $name =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
		$SQL .= qq| and name LIKE "%$name%" |;
	  }
	  if ($query->param("category")) {
	    my $cat = $query->param("category"); $cat =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
		$SQL .= qq| and category = "$cat" |;
	  }
	  if ($query->param("description")) {
	    my $description = $query->param("description"); 
		$description =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
		$SQL .= qq| and description LIKE "%$description%" |;
	  }
	  $SQL .= qq| order by category |;
	  my $sth = closedShop::DatabaseQuery($SQL);
	  
	  my $mark = qq|<div align="left">(P) = Picture Available</font></div>|;
	  if (-e "$closedShop::IMAGEPATH/photoexists.gif") {
	    $mark = qq|<div align="left"><img src="$closedShop::SecureImageURL/photoexists.gif"> 
	    <font color="Red"> = Photo Available</font></div>|;
	  }
	  
	  print qq|$closedShop::font
	  $mark
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="4">
	  $closedShop::font <font size="3"><font color="blue">Search Results</font></th></tr>
	  <tr bgcolor="$closedShop::mc">
	  <td>$closedShop::font<strong>Name</strong></td>
	  <td>$closedShop::font<strong>Category</strong></td>
	  <td>$closedShop::font<strong>Price</strong></td>
	  <td>$closedShop::font<strong>Cart</strong></td>
	  </tr>|;

	  my $k = 0;
	  while (my ($id,$name,$price,$category) = $sth->fetchrow_array()) {
	    my $picturemark = "";
	    if (-e "$closedShop::IMAGEPATH/$id.jpg") {
	      if (-e "$closedShop::IMAGEPATH/photoexists.gif") {
		    if ($closedShop::FileNames{"item"} =~ /https/) {
	          $picturemark = qq|<img src="$closedShop::SecureImageURL/photoexists.gif"> |;
		    }
		    else {
		      $picturemark = qq|<img src="$closedShop::IMAGEURL/photoexists.gif"> |;
		    }
		  }
		  else {
		    $picturemark = qq|<font color="red">(P)</font> |;
		  }
	    }
		my ($intro,$style,$numimages,$status) = closedShop::GetCategoryInfo($category);
		if ($status ne "PRIVATE") {
	      print qq|
	   	  <tr bgcolor="$closedShop::mc"><td>
		  $closedShop::font $picturemark <a 
		  href="$program_name?action=ShowItem&id=$id&quantity=1">$name</a></td>
		  <td>$closedShop::font <a 
	 href="$program_name?action=ShowCategory&category=$category">$closedShop::categories{$category}</a></td>
		  <td>$closedShop::font \$$price</td>
		  <td>$closedShop::font <a href="$closedShop::FileNames{cart}?action=AddProduct&id=$id&quantity=1">Add to Cart</a></td>
		  </tr>  |;	
		  $k++;
		}
	   }  
	
	  if ($k == 0) {
	    print qq|<tr bgcolor="$closedShop::mc"><td colspan="4">
		  $closedShop::font <div align="center">No items matched your search criteria</div>
		  </td></tr>|;
	  }
	
	   print qq|<tr bgcolor="$closedShop::tc"><th colspan="4">
	   $closedShop::font <font size="3"><font color="blue">
	   Your Search Returned $k results</font></th></tr></table>
	   $closedShop::font  <br>
	   <div align="center"><a href="$program_name">Search Again</a></div>|;
	}
	else {
	  print qq|
	  <form action="$program_name" method="POST">
	  <input type="hidden" name="action" value="Search">
	  <input type="hidden" name="selected" value="1">
	  <table width="550" border="1"><tr bgcolor="$closedShop::tc"><th colspan="2">
	  $closedShop::font <font size="3"><font color="blue">Search</font></th></tr>
	    <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font By Name:</td>
	  <td>$closedShop::font <input type="text" name="name" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font By Category:</td>
	  <td>$closedShop::font <select name="category">
	  <option value="">Any Category</option>\n  |;
	  foreach (sort keys %closedShop::categories) {
	    my ($intro,$style,$numimages,$status) = closedShop::GetCategoryInfo($_);
		if ($status ne "PRIVATE") {
	      print qq|<option value="$_">$closedShop::categories{"$_"}</option>\n|;
		}
	  }
	  print qq|
	  </select></td></tr>
	  <tr bgcolor="$closedShop::mc"><td><center>$closedShop::font By Description:</td>
	  <td>$closedShop::font <input type="text" name="description" size="20"></td></tr>
	  
	  <tr bgcolor="$closedShop::mc"><td colspan="2"><center>$closedShop::font
	  <input type="submit" value="Search"></td></tr></form>
	  </table>
	  |;
	}
	closedShop::Footer();
}




