#!/usr/bin/perl 

################################################################################
# 
# PROGRAM:  User.pl - from closedShop - Open Shopping Cart 
#
# RCSID:  $Id$
#
# DESCRIPTION: User accounts
#
# INPUTS:  user input from the web, relational database (mSQL), filesystem
#
# OUTPUTS:  web output, relational database (mSQL)
#
# SPECIAL NOTES:  
#
################################################################################
#
# MODIFICATION HISTORY
#
# 2001-05-03   cbf    Created
# 2001-08-12   cbf    All input validated
################################################################################

#use CGI::Carp(fatalsToBrowser);
use closedShop;
use strict;
use CGI;
use DBI;
use Fcntl ':flock';

closedShop::Initialize();

###############################################################################
## PROGRAM SPECIFIC VARS ##
my $query = new CGI;
my $program_name = "$closedShop::FileNames{user}";
	
my %actions = (EditInfo => "Edit Your Information",
	           Wishlist => "View Wishlist",
			   ViewOrders => "View Orders",
			   ChangePassword => "Change Password"
			   );
################################################################################


Main();
closedShop::Cleanup();

################################################################################
# FUNCTION:  Main
# DESCRIPTION: Find out what the user wants to do and execute the appropriate
#              sub routine
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    Created subroutine
################################################################################
sub Main
{
	my $action = $query->param('action'); 
	$action =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	if ($action eq "Login") {
	  Login();
	}
	elsif ($action eq "NewAccount") {
	  NewAccount();
	}
	elsif ($actions{$action} eq "") { 
	  LoginScreen("Please login to access your account");
	}
	else {
      my ($session,$uid) = closedShop::GetSessionCookie();
	  if (!closedShop::Authenticate($session,"user")) {
	    return LoginScreen("You are not logged in correctly");
	  }
	  eval ($action);
	}
}

###############################################################################
# FUNCTION:  Wishlist
# DESCRIPTION: The user will log in, sending the username and password
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Wishlist
{
	my ($session,$uid) = closedShop::GetSessionCookie();
	my $itemid = $query->param("id"); $itemid =~ tr/0-9//cd;
	if ($query->param("Remove") && $itemid) {
	  my $SQL = qq| select wishlist_ids from $closedShop::sql_tables{"user"}
               where id = "$uid" |;
	  my $sth = closedShop::DatabaseQuery($SQL);
	  my ($wish) = $sth->fetchrow_array();
	  
	  $itemid =~ tr/0-9//cd;
	  $itemid = $itemid . ":";
	  $wish =~ s/$itemid//g;
	  $SQL = qq| update $closedShop::sql_tables{"user"} set wishlist_ids="$wish"
               where id = "$uid" |;
	  $sth = closedShop::DatabaseQuery($SQL);
	}
    return UserScreen("You have successfully removed that item from your 
	wishlist",$uid);
}

###############################################################################
# FUNCTION:  Login
# DESCRIPTION: The user will log in, sending the username and password
################################################################################
# MODIFICATION HISTORY
# 2000-07-24   cbf    Created
##############################################################################
sub Login
{
	my $username = closedShop::ValidateInput($query->param('username'));
	my $password = closedShop::ValidateInput($query->param('password'));
	
	my $SQL	= qq| select id from $closedShop::sql_tables{"user"} where email = "$username" and
	              password = "$password" |;
	my $sth = closedShop::DatabaseQuery($SQL);
	my ($uid) = $sth->fetchrow_array();
	
	# if we pass this condition, the user has logged in, so retrieve information
	if (!$uid) {	
		return LoginScreen("You supplied an incorrect email/password combination");  
	}
	else {	
	  LoginTheUser($uid);
	}
}

################################################################################
# FUNCTION:  ChangePassword
# DESCRIPTION: ChangePassword
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub ChangePassword
{
	my $old = closedShop::ValidateInput($query->param("old"));
	my $new = closedShop::ValidateInput($query->param("new"));
	my $confirm = closedShop::ValidateInput($query->param("confirm"));
	
	my ($session,$uid) = closedShop::GetSessionCookie();
	
	my $SQL = qq| select email from $closedShop::sql_tables{"user"}
               where password = "$old" and id = "$uid" |;
	my $sth = closedShop::DatabaseQuery($SQL);
	if (!$sth->fetchrow_array()) {
	  return UserScreen("Your password is incorrect",$uid);
	}
	elsif ($new ne $confirm) {
	  return UserScreen("Your new passwords do not match",$uid); 
	}
	else {
	  $SQL = qq| update $closedShop::sql_tables{"user"}
              set password="$new" where id = "$uid" |;
	  $sth = closedShop::DatabaseQuery($SQL);
	  return UserScreen("You have successfully updated your password",$uid);
	}
}

################################################################################
# FUNCTION:  EditInfo
# DESCRIPTION: edits info
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub EditInfo
{
	my $name = closedShop::ValidateInput($query->param("name"));
	my $email = closedShop::ValidateInput($query->param("email"));
	my $address = $query->param("address"); $address =~ s/"/'/g;
	my $city = closedShop::ValidateInput($query->param("city"));
	my $state = closedShop::ValidateInput($query->param("state"));
	my $zip = closedShop::ValidateInput($query->param("zip"));
	my $phone = closedShop::ValidateInput($query->param("phone"));
	
	my ($session,$uid) = closedShop::GetSessionCookie();
	
	my $SQL = qq| update $closedShop::sql_tables{"user"}
              set name="$name",address="$address",city="$city",state="$state",
			  zip="$zip",phone="$phone" where id = "$uid" |;
	my $sth = closedShop::DatabaseQuery($SQL);

	$SQL = qq| select email from $closedShop::sql_tables{"user"}
               where email = "$email" and id != "$uid" |;
	$sth = closedShop::DatabaseQuery($SQL);
	if ($sth->fetchrow_array()) {
	  return UserScreen("The email you chose is already in use",$uid);
	}
	else {
	  $SQL = qq| update $closedShop::sql_tables{"user"}
              set email="$email" where id = "$uid" |;
	  $sth = closedShop::DatabaseQuery($SQL);
	  return UserScreen("You have successfully updated your info",$uid);
	}
}

################################################################################
# FUNCTION:  UserScreen
# DESCRIPTION: Displays the admin screen
################################################################################
# MODIFICATION HISTORY
# 2000-07-25   cbf    created
################################################################################
sub UserScreen
{
	my $message = shift;
	closedShop::Header($message);
	$message = "" if ($message eq "nohttp");
	my $uid = shift;
	
	print qq|<br>
	$closedShop::font	<h4>Welcome back to your account at $closedShop::CompanyName.</h4>
	
	<font color="red">$message</font><br>$closedShop::font
	<table border="0" width="550"><tr><td valign="top">
	<table border="0"><tr bgcolor="$closedShop::tc"><td colspan="1"  width="40%" >
	$closedShop::font <font color="blue"><b>Account Options</b></font></td></tr>
	|;
	
	foreach (sort keys %actions) {
	  print qq|<tr><td bgcolor="$closedShop::mc">$closedShop::font
	  <a href="#$_">$actions{$_}</a>	  
	  </td></tr>|;
	}
	
	print qq|
	 <tr><td colspan="1"  bgcolor="$closedShop::tc">
	$closedShop::font &nbsp;</td></tr>
	</table></td><td width="60%" rowspan="2" valign="top">$closedShop::font	|;
	
	 my $SQL = qq| select name,address,city,state,zip,phone,email,wishlist_ids 
                from $closedShop::sql_tables{"user"} where id = "$uid" |;
	 my $sth = closedShop::DatabaseQuery($SQL);
	 my ($name,$address,$city,$state,$zip,
	    $phone,$email,$wishlist_ids) = $sth->fetchrow_array(); 	
	
	print qq|
	<table width="275" border="0"><tr><td colspan="1"  bgcolor="$closedShop::tc">
	$closedShop::font <font color="blue"><b>
	<a name="EditInfo">User Information - Change</a></b></font></td></tr>
	<form action="$program_name" method="POST">
	<input type="hidden" name="action" value="EditInfo">
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Name</b>: </font><font size="1">
	<input type="text" name="name" value="$name" size="20"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Email</b></font>: <font size="1"><input type="text" name="email" value="$email" size="20"> </td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Address</b></font>: <font size="1"><textarea rows="2" name="address" cols="20">$address</textarea> </td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>City</b></font>: <font size="1"><input type="text" name="city" value="$city" size="20"> </td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>State</b></font>: <font size="1"><input type="text" name="state" value="$state" size="3"> </td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Zip</b></font>: <font size="1"><input type="text" name="zip" value="$zip" size="12"> </td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<b>Phone</b></font>:<font size="1"> <input type="text" name="phone" value="$phone" size="20"> </td></tr>	  
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font<center><b><input type="submit" value="Edit Information" name="EditInfo"></b></center></td></tr>	
    <tr><td colspan="2" bgcolor="$closedShop::tc">
	$closedShop::font &nbsp;</td></tr>  
	 </form></table>
	 </td></tr><tr><td valign="top"  width="40%" >
	 <table border="0" width="100%"><tr><td colspan="2" bgcolor="$closedShop::tc">
	 <form action="$program_name" method="POST">
	<input type="hidden" name="action" value="ChangePassword">
	$closedShop::font <font color="blue"><b><a name="ChangePassword">Change Password</a></b></font></td></tr>
	 <tr bgcolor="$closedShop::mc">
	 <td>$closedShop::font<b>Old Password</b></td><td>$closedShop::font
	<font size="1"><input type="password" name="old" value="" size="12"> </td>
	</tr>
	<tr  bgcolor="$closedShop::mc">
	 <td>$closedShop::font<b>New Password</b></td><td>$closedShop::font
	<font size="1"> <input type="password" name="new" value="" size="12"> </td>
	</tr>
	<tr  bgcolor="$closedShop::mc">
	
	 <td>$closedShop::font<b>Confirm New Password</b></td><td>$closedShop::font
	<font size="1"> <input type="password" name="confirm" value="" size="12"> </td>
	</tr> 

	<tr bgcolor="$closedShop::mc"><td colspan="3">$closedShop::font<center>
	<b><input type="submit" value="Change Password" name="ChangePassword"></b></center></td></tr></form>
 <tr><td colspan="2" bgcolor="$closedShop::tc">
	$closedShop::font &nbsp;</td></tr>
	 </table></td></tr>
	 
	 
	 <tr><td valign="top"  width="100%" colspan="2">$closedShop::font
	 <table border="0" width="100%"><tr><td colspan="6" bgcolor="$closedShop::tc">
	$closedShop::font <font color="blue"><b><a name="ViewOrders">Order List</a></b></font></td></tr>
	 <tr bgcolor="$closedShop::mc">
	 <td>$closedShop::font<b>ID</b></td>
	 <td>$closedShop::font<b>Items</b></td>
	  <td>$closedShop::font<b>Date Bought</b></td>
	  <td>$closedShop::font<b>Date Processed</b></td>
	  <td>$closedShop::font<b>Status</b></td>
	  <td>$closedShop::font<b>Total</b></td></tr>|;
	  
	  $SQL = qq| select id,item_ids,DATE_FORMAT(date_bought,"%M %d %Y"),
	  				DATE_FORMAT(date_processed,"%M %d %Y"),complete,
	                total from $closedShop::sql_tables{"order"} 
	                where complete="Y" and user_id="$uid" order by date_bought |;

	  $sth = closedShop::DatabaseQuery($SQL);
	  my $k = 0;
 	  while (my ($oid,$iids,$date_bought,$date_processed,$complete,$total) = $sth->fetchrow_array()) {
        $k++;
		my @ary = split(/~/,$iids);  
		my $item_text = "";
		foreach (@ary) {
	  	  my ($iid,$qty) = split(/\|/,$_);
	  	  my $SQL2	= qq| select name,category from $closedShop::sql_tables{"item"} where
				      id="$iid" |;
	      my $sth2 = closedShop::DatabaseQuery($SQL2);
	      my ($name,$cat) = $sth2->fetchrow_array();
		  $item_text .= qq|<li>$qty x <a href="$closedShop::FileNames{item}?action=ShowItem&id=$iid">$name</a> from 
		  <a href="$closedShop::FileNames{item}?action=ShowCategory&category=$cat">$closedShop::categories{$cat}</a></li>
		  |;
		}
		
		
		if ($date_processed)  { $complete = "Completed"; }
		else                   { $complete = "Pending"; }
		if ($date_processed eq "") { $date_processed = "--"; }
		  print qq|
		    <tr bgcolor="$closedShop::mc">
			<td>$closedShop::font $oid</td>
			<td>$closedShop::font $item_text</td>
			<td>$closedShop::font $date_bought</td>
			<td>$closedShop::font $date_processed</td>
			<td>$closedShop::font $complete</td>
			<td>$closedShop::font \$ $total</td></tr>\n |;
	  }
	 
	 if ($k == 0) {
	   print qq|<tr><td colspan="6" bgcolor="$closedShop::mc">
	$closedShop::font <div align="center"><b>You have not placed any orders</b></div></font></td></tr>|;
	 }
	 
	 print qq| <tr><td colspan="6" bgcolor="$closedShop::tc">
	$closedShop::font &nbsp;</td></tr>
	 </table>
	 </td></tr>
	
	  <tr><td valign="top"  width="100%" colspan="2">$closedShop::font
	 <table border="0" width="100%"><tr><td colspan="6" bgcolor="$closedShop::tc">
	$closedShop::font <font color="blue"><b><a name="Wishlist">WishList</a></b></font></td></tr>
	 <tr bgcolor="$closedShop::mc">
	 <td>$closedShop::font<b>Item Name</b></td>
	 <td>$closedShop::font<b>Category</b></td>
	  <td>$closedShop::font<b>Shipping Price</b></td>
	  <td>$closedShop::font<b>Item Price</b></td>
	  <td>$closedShop::font<b>Add to Cart</b></td>
	  <td>$closedShop::font<b>Remove from Wishlist</b></td>
	  </tr>|;
	  
	my @wish = split(/:/,$wishlist_ids);

	$k = 0;
	foreach (@wish) {
	  $k++;
	  my $SQL2	= qq| select name,category,shipping_price,
	  price from $closedShop::sql_tables{"item"} where id="$_" |;
	  my $sth2 = closedShop::DatabaseQuery($SQL2);
	  my ($n,$c,$s,$p) = $sth2->fetchrow_array();
	  print qq|
		<tr bgcolor="$closedShop::mc">
	 	<td>$closedShop::font<a href="$closedShop::FileNames{item}?action=ShowItem&id=$_">$n</a></td>
	 	<td>$closedShop::font $closedShop::categories{$c}</td>
	  	<td>$closedShop::font \$$s</td>
	  	<td>$closedShop::font \$$p</td>
	  	<td>$closedShop::font 
		<a href="$closedShop::FileNames{cart}?action=AddProduct&id=$_&quantity=1">Add to Cart</a></td>
		<td>$closedShop::font 
		<a href="$program_name?action=Wishlist&Remove=1&id=$_">Remove from Wishlist</a></td></tr>
		  |;
	 }
	 if ($k == 0) {
	   print qq|<tr><td colspan="6" bgcolor="$closedShop::mc">
	$closedShop::font <div align="center"><b>You have no items in your wishlist</b></div></font></td></tr>|;
	 }
	 
	 print qq|
	 <tr><td colspan="6" bgcolor="$closedShop::tc">
	$closedShop::font &nbsp;</td></tr>
	 </table>
	 </td></tr>
	 </table>
	</center></div>|;
	
	closedShop::Footer();
}

################################################################################
# FUNCTION:  LoginScreen
# DESCRIPTION: login screen for admin
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub LoginScreen
{
	my ($session,$uid) = closedShop::GetSessionCookie();
	if (closedShop::Authenticate($session,"user")) {
	  return UserScreen("",$uid);
	}

    my $mesg = $query->param("message") || $_[0];
	closedShop::Header();
	print qq| <form action="$program_name" method="POST">
	<input type="hidden" name="action" value="Login">
	$closedShop::font
	<font color="red">$mesg</font>
	<table border="0"><tr bgcolor="$closedShop::tc"><td colspan="2">$closedShop::font<font color="blue">
	<div align="center"><b>Login</b></div></td></tr><tr bgcolor="$closedShop::mc"><td>$closedShop::font
	Email Address: </td><td><input type="text" name="username" size="25"></td></tr>
	<tr bgcolor="$closedShop::mc"><td>$closedShop::font
	Password: </td><td><input type="password" name="password" size="25"></td></tr>
	<tr bgcolor="$closedShop::mc"><td  colspan="2">
	<div align="center"><input type="submit" name="admin_login" value="Login"></div>
	</form>
	
	<tr bgcolor="$closedShop::mc"><td colspan="2">&nbsp;</td></tr>
	  
	  <form action="$program_name" method="POST">
	<input type="hidden" name="action" value="NewAccount">
	  <tr bgcolor="$closedShop::mc">
	  <td colspan="2"><div align="center">$closedShop::font<strong>Create Account</strong><br>
	  Please fill required fields</div></td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Name: </td>
	  <td>$closedShop::font <input type="name" name="name" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Email Address: </td>
	  <td>$closedShop::font <input type="text" name="email2" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Password: </td>
	  <td>$closedShop::font <input type="password" name="password2" size="20"></td></tr>
  	  <tr bgcolor="$closedShop::mc"><td>$closedShop::font Confirm Password: </td>
	  <td>$closedShop::font <input type="password" name="confirm" size="20"></td></tr>
	  <tr bgcolor="$closedShop::mc"><td colspan="2"><div align="center">$closedShop::font
	  <input type="submit" name="new" value="Create Account"></div></td></tr>
	  </table>
	<br></form>
	|;
	
	closedShop::Footer();
}

################################################################################
# FUNCTION:  NewAccount
# DESCRIPTION: NewAccount
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub NewAccount
{
    my $name = $query->param("name"); $name =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
    my $email = $query->param("email2"); $email =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	my $password = $query->param("password2"); $password =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	my $confirm = $query->param("confirm"); $confirm =~ tr/a-zA-Z0-9-_ .,:;'&$#@!*()?-//cd;
	
	if ($password ne $confirm) {
	  return LoginScreen("Your passwords do not match");
	}
	if (!$name || !$email || !$password) {
	  return LoginScreen("Please fill out all fields");	
	}
	my $SQL	= qq| select email from $closedShop::sql_tables{"user"} 
	             where email="$email" |;
	my $sth = closedShop::DatabaseQuery($SQL);
	my ($temail) = $sth->fetchrow_array();
	if ($temail) {
	  return LoginScreen("The email address has already been taken.");	
	}
	
	$SQL = qq| insert into $closedShop::sql_tables{"user"} 
	           set email="$email",name="$name",password="$password" |;
    $sth = closedShop::DatabaseQuery($SQL);
	my $new_id = $sth->{'mysql_insertid'};
	
	LoginTheUser($new_id);
}

################################################################################
# FUNCTION:  LoginTheUser
# DESCRIPTION: process screen
################################################################################
# MODIFICATION HISTORY
# 2000-05-19   cbf    Validated input, cleaned up sql statements
################################################################################
sub LoginTheUser
{
      my $uid = shift;
	  my $session = closedShop::CreateSession();
	  my $SQL = qq| update $closedShop::sql_tables{"user"} set session="$session" where id = "$uid" |;
	  my $sth = closedShop::DatabaseQuery($SQL);
	  closedShop::SetSessionCookie($session,$uid);
	  UserScreen("nohttp",$uid);
}









