#!/usr/bin/perl
#			   -------------
#				   Links
#			   -------------
#			   Links Manager
#
#		 File: nph-build.cgi
#  Description: Builds a set of HTML pages from the template directory. This is a
#			   non parsed header script, and should display the output directly as it may
#			   take quite a while to perform. It can also be called from the
#			   command line or via a cron routine. Read the README for more details.
#	   Author: Alex Krohn
#		Email: alex@gossamer-threads.com
#		  Web: http://www.gossamer-threads.com/
#	  Version: 1.11
#
# COPYRIGHT NOTICE:
#
# Copyright 1997 Gossamer Threads Inc.  All Rights Reserved.
#
# This program is being distributed as shareware.  It may be used and
# modified free of charge for personal, academic, government or non-profit
# use, so long as this copyright notice and the header above remain intact.
# Any commercial use should be registered.  Please also send me an email,
# and let me know where you are using this script. By using this program 
# you agree to indemnify Gossamer Threads Inc. from any liability.
#
# Selling the code for this program without prior written consent is
# expressly forbidden.  Obtain written permission before redistributing this
# program over the Internet or in any other medium.  In all cases
# copyright and header must remain intact.
#
# Please check the README file for full details on registration.
# =====================================================================


# Required Librariers
# --------------------------------------------------------
eval {
	($0 =~ m,(.*)/[^/]+,)   && unshift (@INC, "$1");	# Get the script location: UNIX /
	($0 =~ m,(.*)\\[^\\]+,) && unshift (@INC, "$1");	# Get the script location: Windows \

    require "links.cfg";				# Change this to full path to links.cfg if you have problems.
	require "$db_lib_path/db.pl";
	require "$db_lib_path/site_html.pl";
};
if ($@) {
	print "Content-type: text/plain\n\n";
	print "Error including libraries: $@\n";
	print "Make sure they exist, permissions are set properly, and paths are set correctly.";
	exit;
}

# ========================================================

eval { &main; };							# Trap any fatal errors so the program hopefully 
if ($@) { &cgierr("fatal error: $@"); }	 # never produces that nasty 500 server error page.
exit;   # There are only two exit calls in the script, here and in in &cgierr. 

sub main {
# --------------------------------------------------------  
# Our Global variables that will hold all the link information:	
	local (%category, %subcategories, @links, @new_links, @cool_links, %stats, $grand_total);

	$| = 1;	 # Flush output.

# Determine if we are printing to command line, or to browser.
	$use_html = 0;
	if ($ENV{'REQUEST_METHOD'}) { $use_html = 1; }

# Print full headers for nph scripts.
	$nph = 1;
	&html_print_headers 		if ($use_html);

# Print HTML Header
	$use_html ?
		print qq|<html><head><tittle>Links Manager: Building Pages</title></head>
				 <BASE TARGET="_top">
				 <BODY BGCOLOR=#FFFFFF><H2><TT>Building Pages</TT></H2>
				 <PRE>| :
		print qq|Building Pages\n|;
	print "Pages built on " . $date . " at " . $time . "\n";
	print "--------------------------------------------------------\n\n";

# Back up Database..
if (! (-e "$db_backup_path/$date.db")) {
	print "Backing up database to $db_backup_path/$date.db . . .\n";
	use File::Copy;
	copy ($db_file_name, "$db_backup_path/$date.db") or &cgierr ("unable to make backup $db_backup_path/$date.db! Reason: $!");
	print "Done.\n\n";
}
else {
	print "Backup Database already exists for today.. skipping..\n\n";
}

# Update New and Popular Records..
	print "Updating New and Popular Records . . .\n";
	&build_update_newpop;
	print "Done.\n\n";

# Rebuild URL Index (This file is auto-generated, you will never need to touch it!	
	print "Building URL Index . . .\n";
	&build_url_index;
	print "Done.\n\n";
	
# Load Category Information
	print "Loading Category Information . . .\n";
	&build_category_information;
	print "Done.\n\n";
	
# Generate some stats for future pages...	
	print "Gathering Category and Link information . . .\n";
	&build_stats;
	print "Done\n\n";
	
# Create Home Page
	$use_html ?
		print qq|Building <A HREF="$build_root_url/$build_index">Home Pages</A> . . .\n| :
		print qq|Building Home Page . . .\n|;
	&build_home_page;
	print "\tDone\n\n";
	
# Create What's New Page	
	$use_html ?
		print "Building <A HREF=\"$build_new_url/$build_index\">What's New</A> Page . . .\n" :
		print "Building What's New Page . . .\n";
	&build_new_page;
	print "Done\n\n";

# Create What's Cool Page
	$use_html ?
		print "Building <A HREF=\"$build_cool_url/$build_index\">What's Cool</A> Page . . .\n" :
		print "Building What's Cool Page . . .\n";
	&build_cool_page;
	print "Done\n\n";
	
# Create Category Pages
	print "Building Category Pages . . .\n";
	&build_category_pages;
	print "Done\n\n";
	
# We are finished! 	
	print "Pages Built!";
	print "</PRE></BODY></HTML>" if ($use_html);
}

sub build_update_newpop {
# --------------------------------------------------------
# This routines updates the database, and marks new records new,
# old records old, popular records popular, and unpopular records,
# unpopular.

	my (@lines, $line, %hits, @fields, $days, %new_records, @popular, $cutoff);

	open (URL, "<$db_url_name") or &cgierr("unable to open url index: $db_url_name. Reason: $!");
		@lines = <URL>;
	close (URL);

# Go through the old url database and find out how many hits each record
# now has. Store the hits in a hash %hits accessed by id.	
	my ($id, $url, $hits, @rest);
	LINE: foreach $line (@lines) {
		next LINE if ($line =~ /^#/);
		next LINE if ($line =~ /^\s*$/);
		chomp $line;

		($id, $url, $hits, @rest) = split (/\Q$db_delim\E/, $line);
		$hits{$id} = $hits;
	}   

# Go through the database and calculate which records are new and build an 
# array @popular of the range of hits each link has. We will eventually
# sort that array to determine the cutoff value.
	open (DB, "<$db_file_name") or &cgierr("unable to open database: $db_file_name. Reason: $!");
		@lines = <DB>;
	close DB;
	
	LINE: foreach $line (@lines) {
		next LINE if ($line =~ /^#/);
		next LINE if ($line =~ /^\s*$/);
		chomp $line;

		@fields = &split_decode ($line);

		if (!$fields[$db_modified]) {
			print "Warning! Bad Line (no date!): $line\n";
			next LINE;
		}

# Calculate how many day's old this record is using &days_old, and then
# mark it new if it passes the cutoff.
		$days = &days_old($fields[$db_modified], $date);
		($days < $db_new_cutoff) &&	($new_records{$fields[$db_cols[$db_key]]} = 1);

# If this is a link we've seen in url.db, then we add it onto the popular mark,
# otherwise we add the value found in the database.
		if ($hits{$fields[$db_hits]}) { push (@popular, $fields[$db_hits]); }
		else {			  				push (@popular, $fields[$db_hits]); }
	}

# Sort the list of popular records, and make a cutoff mark. We also limit the
# cutoff to at least two, so everything isn't marked cool on new databases.	
	@popular = sort { $b <=> $a } @popular;
	$cutoff = $popular[int($db_popular_cutoff * $#popular)];
	if ($cutoff < 2) { $cutoff = 2; }
	print "\tWhat's New Cutoff: $db_new_cutoff days\n";
	print "\tPopular Cutoff: $cutoff hits\n";

# Now we go through the database one more time to store the popular results
# and the new record result.
	LINE: foreach $line (@lines) {
		next LINE if ($line =~ /^#/);
		next LINE if ($line =~ /^\s*$/);
		chomp $line;
			
		@fields = &split_decode ($line);
		$id = $fields[0];

# Store the new number of hits.		
		($hits{$id}) &&	($fields[$db_hits] = $hits{$id});

# Check to see if the record is popular...	 
		if ($fields[$db_hits] >= $cutoff) {
			print "\tUpdating record: $fields[0], marking as popular ($fields[$db_hits]).\n";
			$fields[$db_ispop] = "Yes";
		}
		else {
			$fields[$db_ispop] = "No";
		}

# Check to see if the record is new...	  
		if ($new_records{$fields[$db_cols[0]]}) {
			print "\tUpdating record: $fields[$db_cols[0]], marking as new.\n";   
			$fields[$db_isnew] = "Yes";
		}
		else {
			$fields[$db_isnew] = "No";
		}		
		$output .= &join_encode (&array_to_hash(0, @fields));
	}

# Save this information!
	open (DB, ">$db_file_name") or &cgierr("error in validate_records, unable to open db file: $db_file_name. Reason: $!");
	flock(DB, $LOCK_EX) unless (!$db_use_flock);
		print DB $output;
	close DB;
}

sub build_url_index {
# --------------------------------------------------------
# This rountine builds a lookup index that just contains the 
# database key, the url and the number of hits. This is used 
# so that the database does not have to be modified every time
# someone clicks on a link, and yet hits can still be tracked.

	my (@lines, $line, @fields, $time);
	$time = time();
	
	open (DB, "<$db_file_name") or &cgierr("unable to open database: $db_file_name. Reason: $!");
		@lines = <DB>;
	close DB;
	
	open (URL, ">$db_url_name") or &cgierr("unable to open url index: $db_url_name. Reason: $!");
	foreach $line (@lines) {
		next LINE if ($line =~ /^#/);   # Skip Comment Lines.
		next LINE if ($line =~ /^\s*$/); # Skip Blank Lines.
		chomp $line;					# Remove new line char.

		@fields = &split_decode ($line);
		
		print URL "$fields[0]$db_delim$fields[$db_url]$db_delim$fields[$db_hits]$db_delim$time$db_delim\n";
	}
	close URL;
}   

sub build_category_information {
# --------------------------------------------------------
# This routine loads the category information into memory and
# does some grouping.

	my (@lines, $line, @values, $name, @base);

	open (DB, "<$db_category_name") or &cgierr("unable to open database: $db_file_name. Reason: $!");
		@lines = <DB>;
	close DB;
	
	LINE: foreach $line (@lines) {
		next LINE if ($line =~ /^#/);
		next LINE if ($line =~ /^\s*$/);
		chomp $line;
	
		@values = &split_decode ($line);
		
		$name = $values[$db_main_category];

# We store the category information in a globally accessable hash
# %category, indexed by category name.
		@{$category{$name}} = @values;
		
# We also initilize the days_old variable to Jan 1. 1990 so that if
# this category doesn't have any links, we still have a valid date
# for it.
		$stats{$name}[1] = "1-Jan-1990";
		$stats{$name}[0] = 0;

# We now figure out if this category is a subcategory of something,
# and if so, store it in the hash of arrays %subcategories.
		(@base) = split (/\//, $name);
		pop @base; $base = join ("/", @base);
		if ($base) {
			push (@{$subcategories{$base}}, $name);
		}
	}
}

sub build_stats {
# --------------------------------------------------------
# This routine does a lot of the messy work. It builds globally accessible
# arrays of new_links and cool_links. It finds out how many links are in each
# category, and whether a category contains a new/modified link.

	my ($line, @lines, @fields, $category, @categorylist, $depth);
	open (DB, "<$db_file_name") or &cgierr("unable to open database: $db_file_name. Reason: $!");
		@lines = <DB>;
	close DB;

# Set global variable $grand_total to the total number of links (quick way to find out is
# just count the number of lines in the database.. Might be off a bit if you have
# comments. =)
	$grand_total = $#lines + 1;
	LINE: foreach $line (@lines) {
		next LINE if ($line =~ /^#/);   # Skip Comment Lines.
		next LINE if ($line =~ /^\s*$/); # Skip Blank Lines.
		chomp $line;					# Remove new line char.

		@fields = &split_decode ($line);
		$category = $fields[$db_category];
		
# Add the link to the list of links.		
		push (@{$links{$category}}, @fields);

# Add the link to the list of new links if it is new.
		push (@{$new_links{$category}}, @fields) if ($fields[$db_isnew] eq "Yes");

# Add the link to the list of cool links if it is popular.
		push (@{$cool_links{$category}}, @fields) if ($fields[$db_ispop] eq "Yes");

# Calculate the stats: the number of links and the newest link.			 
		@categorylist = split (/\//, $category);
		$depth = $#categorylist;		

# This adds one to the total of each category above the current category,
# and also marks any above categories new, if this link is new.
		for ($i = 0; $i <= $depth; $i++) {  
			$stats{$category}[0]++;
			if ((!$stats{$category}[1]) || &compare_dates($fields[$db_modified], $stats{$category}[1])) {
				$stats{$category}[1] = $fields[$db_modified];
			}
			pop (@categorylist);
			$category = join("/", @categorylist);
		}	   
	}   
	
# Now we have to sort the links and categories..
	foreach $link ( keys %links ) {
		@{$links{$link}} = &build_sorthit (@{$links{$link}});
	}
	foreach $cat ( keys %subcategories ) {
		@{$subcategories{$cat}} = sort @{$subcategories{$cat}};
	}	
}   

sub build_category_pages {
# --------------------------------------------------------
# This routine builds all the category pages. Each category uses
# the same template which is defined in &site_html_category.

	my ($cat, $url, $dir, @related, $relation, $page_num, $next_page, $numlinks);
	local ($category, $links, $title_linked, $title, $total, $category_name, $category_name_escaped);
	local ($description, $related, $meta_name, $meta_keywords, $header, $footer, $next, $prev);

# Go through each category and build the appropriate page.	
	CATEGORY: foreach $cat (sort keys %category) {
		next CATEGORY if ($cat =~ /^\s*$/);		# How'd that get in here? =)
		
		$url = "$build_root_url/" . &urlencode($cat) . "/";		
		$use_html ?
			print qq|Building Category: <A HREF="$url">$cat</A>\n| :
			print qq|Building Category: $cat\n|;
		print "\tSubcategories: " . ($#{$subcategories{$cat}} + 1) 		   . "\n";
		print "\tLinks: " 		  . (($#{$links{$cat}}+1) / ($#db_cols+1)) . "\n";

# Let's make sure the directory exists, build it if it doesn't.
		$dir = &build_dir ($cat);
		print "\tDirectory: $dir\n";
		print "\tFilename : $dir/$build_index\n";

# We set up all the variables people can use in &site_html.pl.			
		($description, $related, $meta_name, $meta_keywords, $header, $footer) = @{$category{$cat}}[2..7];
# Calculate the related entries and put in a <LI> list.
		@related = split(/\Q$db_delim\E/, $related); $related = "";
		foreach $relation (@related) {
			$related .= qq|<li><a href="$build_root_url/|;
			$related .= &urlencode($relation);
			$related .= qq|/$build_index">|;
			$related .= &build_clean($relation);
			$related .= "</li>";
		}

# Get the header and footer from file if it exists, otherwise assume it is html.
		if ($header && (length($header) < 20) && ($header !~ /\s+/) && (-e "$db_header_path/$header")) {
			open (HEAD, "<$db_header_path/$header") or &cgierr ("Unable to open header file: $db_header_path/$header. Reason: $!");
			$header = "";			 
			while (<HEAD>) {
				$header .= $_;
			}
			close HEAD;
		}
		if ($footer && (length($footer) < 20) && ($footer !~ /\s+/) && (-e "$db_footer_path/$footer")) {
			open (FOOT, "<$db_footer_path/$footer") or &cgierr ("Unable to open footer file: $db_footer_path/$footer. Reason: $!");
			$footer = "";
			while (<FOOT>) {
				$footer .= $_;
			}
			close FOOT;
		}			
		$title_linked 			= &build_linked_title ($cat);
		$title 					= &build_unlinked_title ($cat);
		$total 					= ($#{$links{$cat}} + 1) / ($#db_cols + 1);
		$category_name 			= $cat;
		$category_name_escaped 	= &urlencode ($cat);
		$category_clean 		= &build_clean ($cat);

# Store all the category html info in $category.
		if ($#{$subcategories{$cat}} >= 0) {
			$category = &site_html_print_cat (@{$subcategories{$cat}});
		}
		else {
			$category = "";
		}

# If we are spanning pages, we grab the first x number of links and build
# the main index page. We set $numlinks to the remaining links, and we remove
# the links from the list.
		$numlinks = ($#{$links{$cat}} + 1) / ($#db_cols + 1);
		$next = $prev = $links = "";
		if (($numlinks > $build_links_per_page) && $build_span_pages) {
			$page_num = 2;
			$next  = $url . "more$page_num.html";		
			for ($i = 0; $i < $build_links_per_page; $i++) {
				%tmp = &array_to_hash ($i, @{$links{$cat}});		
				$links .= &site_html_link (%tmp);
			}
			@{$links{$cat}} = @{$links{$cat}}[(($#db_cols+1)*$build_links_per_page) .. $#{$links{$cat}}];
			$numlinks = ($#{$links{$cat}}+1) / ($#db_cols + 1);
		}
# Otherwise we either only have less then x number of links, or we are not
# splitting pages, so let's just build them all.
		else {
			for ($i = 0; $i < $numlinks; $i++) {
				%tmp = &array_to_hash ($i, @{$links{$cat}});		
				$links .= &site_html_link (%tmp);
			}
		}
		

# Create the main page.
		open (CAT, ">$dir/$build_index") or &cgierr ("unable to open category page: $dir/$build_index. Reason: $!");
			print CAT &site_html_category;
		close CAT;

# Then we go through the list of links and build on the remaining pages.
		$prev = $url if ($build_span_pages);		
		while ($next && $build_span_pages) {
			if ($numlinks > $build_links_per_page) {
				$next_page = $page_num+1;
				$next = $url . "more$next_page.html";
			}
			else {
				$next = "";
			}
			$links = "";
			LINK: for ($i = 0; $i < $build_links_per_page; $i++) {
				%tmp = &array_to_hash ($i, @{$links{$cat}});
				last LINK if ($tmp{$db_key} eq "");
				$links .= &site_html_link (%tmp);
			}
			$title_linked 			= &build_linked_title ("$cat/Page_$page_num/");
			$title 					= &build_unlinked_title ("$cat/Page_$page_num/");
			
			$use_html ?
				print qq|\tSubpage  : <A HREF="|, $url, qq|more$page_num.html">$dir/more$page_num.html</A>\n| :
				print qq|\tSubpage  : $dir/more$page_num.html\n|;
			open (CAT, ">$dir/more$page_num.html") or &cgierr ("unable to open category page: $dir/index$page_num.html. Reason: $!");
				print CAT &site_html_category;
			close CAT;

			@{$links{$cat}} = @{$links{$cat}}[(($#db_cols+1)*$build_links_per_page) .. $#{$links{$cat}}];
			$numlinks = ($#{$links{$cat}}+1) / ($#db_cols + 1);
			
			$prev = $url . "more$page_num.html";
			$page_num++;			
		}			
		print "\n";
	}
}

sub build_home_page {
# --------------------------------------------------------
	my ($subcat, @rootcat);
	local ($total);
		
# Check to see which categories are off of the root.
	foreach $subcat (sort keys %category) {
		if ($subcat =~ m,^([^/]*)$,) {
			push (@rootcat, $subcat);
		}
	}
	print "Subcategories: "; print $#rootcat+1; print "\n"; 
	print "Total Links: $grand_total\n";
	print "\tOpening page: $build_root_path/$build_index\n";
			
	open (HOME, ">$build_root_path/$build_index") or &cgierr ("unable to open home page: $build_root_path/$build_index. Reason: $!");
		$category 	= &site_html_print_cat (@rootcat) if ($#rootcat >= 0);
		$total 		= $grand_total;
		print HOME &site_html_home;
	close HOME;
	print "\tClosing page.\n";  
}

sub build_new_page {
# --------------------------------------------------------
# Creates a "What's New" page. Set $build_span_pages to 1 in links.cfg
# and it will create a seperate page for each date.
#
	my (%link_output, $category_clean, $long_date, $category, $date, 
		$number_links, $main_link_results, $main_total, %span_totals);
	local ($total, $link_results, $title_linked);  

# Let's build the What's New directory.	
	if ($build_new_path =~ m,^$build_root_path/(.*)$,) {
		&build_dir ($1);
	}
	
# Now we go through all the new_links (which are organized by category), and
# build the html in array indexed by date then category. 
	$total = 0;	
	CATEGORY: foreach $category (sort keys %new_links) {			
		LINK: for ($i = 0; $i < ($#{$new_links{$category}}+1) / ($#db_cols + 1); $i++) {
			$total++;
			%tmp = &array_to_hash ($i, @{$new_links{$category}});
			${$link_output{$tmp{'Date'}}}{$category} .= &site_html_link (%tmp) . "\n";
			$span_totals{$tmp{'Date'}}++;
		}
	}
	
# Then we go through each date, and build the links for that date. If we are spanning
# pages, we will create a seperate page for each date and need to set up a few other 
# variables (like title and total). We will also want to reset links_results each time.
	DATE: foreach $date (sort { &date_to_unix($b) <=> &date_to_unix($a) } keys %link_output) {
		$long_date 	  = &long_date ($date);
		if ($build_span_pages) {		
			$link_results = "";			
			$total 		  = $span_totals{$date};
			$title_linked = &build_linked_title ("New/$date");
		}
		else {
			$link_results .= "<p><strong>$long_date</strong>\n<blockquote>";
		}
		CATEGORY: foreach $category (sort keys %{$link_output{$date}}) {
			$category_clean = &build_clean ($category);
			$link_results .= qq|<P><A HREF="$build_root_url/$category/$build_index">$category_clean</A>\n|;
			$link_results .= ${$link_output{$date}}{$category};
		}

# Crete the new page, and do a bit of HTML work for the main page.
		if ($build_span_pages) {
			open (NEW, ">$build_new_path/$date.htm") or cgierr ("unable to open what's new page: $build_new_path/$build_index. Reason: $!");
				$use_html ?
					print qq|\tNew Links for <a href="$build_new_url/$date.htm">$date</a>: $total\n| :
					print qq|\tNew Links for $date: $total\n|;
				print NEW &site_html_new;
			close NEW;
			$main_link_results .= qq|<li><a href="$build_new_url/$date.htm">$long_date</a> ($total).|;
			$main_total += $total;
		}
		else {
			$link_results .= "</blockquote>";
		}
	}
	
	if ($build_span_pages) {
		$link_results = "<ul>$main_link_results</ul>";		
		$total = $main_total;
	}
	$title_linked = &build_linked_title ("New");

# Build the main What's New page.
	open (NEW, ">$build_new_path/$build_index") or cgierr ("unable to open what's new page: $build_new_path/$build_index. Reason: $!");
		print "\tTotal New Links: $total\n";
		print NEW &site_html_new(@new_links);
	close NEW;
}

sub build_cool_page {
# --------------------------------------------------------
# Creates a "What's Cool" page.

	local ($total, $percent, $link_results, $title_linked);
	my (%link_output, $category_clean);
	
	if ($build_cool_path =~ m,^$build_root_path/(.*)$,) {
		&build_dir ($1);
	}
	
	$total = 0;
	CATEGORY: foreach $category (sort keys %cool_links) {		   
		LINK: for ($i = 0; $i < ($#{$cool_links{$category}}+1) / ($#db_cols + 1); $i++) {
			$total++;
			%tmp = &array_to_hash ($i, @{$cool_links{$category}});
			$link_output{$category} .= &site_html_link (%tmp) . "\n";
		}
	}
	foreach $category (sort keys %cool_links) {
		$category_clean = &build_clean ($category);
		$link_results .= qq|<P><A HREF="$build_root_url/$category/$build_index">$category_clean</A>\n|;
		$link_results .= $link_output{$category};
	}
	$title_linked = &build_linked_title ("Cool");
	open (COOL, ">$build_cool_path/$build_index") or cgierr ("unable to open what's cool page: $build_cool_path/$build_index. Reason: $!");
		print "\tCool Links: $total\n";
		$percent = ($db_popular_cutoff * 100);
		print COOL &site_html_cool(@cool_links);
	close COOL;
}


sub build_dir {
# --------------------------------------------------------
# Verifies that all neccessary directories have been created 
# before we create the category file. Takes as input, the category
# to verify, and returns the full directory path.

	my ($input) = $_[0];
	my ($dir, $path) = "";
	my (@dirs) = split (/\//, $input);
	
	foreach $dir (@dirs) {
		$path .= "/$dir";
		&build_check_dir ("$build_root_path$path");
		if (! (-e "$build_root_path$path")) {
			print "\tMaking Directory ($build_dir_per): '$build_root_path$path' ...";
			if (mkdir ("$build_root_path$path", "$build_dir_per")) {;
				print "Made. CHMOD $build_dir_per ...";
				if (chmod ("$build_dir_per", "$build_root_path$path")) {;
					print "Done.";
				}
				else { print "CHMOD $build_dir_per failed! Reason: $!."; }
			}
			else { print "mkdir failed! Reason: $!."; }
			print "\n";
		}
	}
	return "$build_root_path$path";
}

sub build_linked_title {
# --------------------------------------------------------
# Returns a string of the current category broken up 
# by section, with each part linked to the respective section.

	my ($input) = $_[0];
	my (@dirs, $dir, $output, $path, $last);

	@dirs = split (/\//, $input);
	$last = &build_clean(pop @dirs);
	
	$output = qq| <A HREF="$build_root_url/">Top</A> :|;
	foreach $dir (@dirs) {
		$path .= "/$dir";
		$dir = &build_clean ($dir);
		$output .= qq| <A HREF="$build_root_url$path/">$dir</A> :|;
	}
	$output .= " $last";
	
	return $output;
}

sub build_unlinked_title {
# --------------------------------------------------------
# Returns a string of the current category broken up by section.
# Useful for printing in the title.

	my ($input) = $_[0];
	my (@dirs, $dir, $output);

	@dirs = split (/\//, $input);
	
	foreach $dir (@dirs) {
		$dir = &build_clean ($dir);
		$output .= " $dir:";
	}
	chop ($output);	 
	return $output;
}

sub build_check_dir {
# --------------------------------------------------------
# Checks the directory before we create it to make sure there 
# are no funncy characters in it.

	my ($input) = $_[0];

	if (!($input =~ m,^[\w\d/_\Q$build_allow_char\E]+$,)) {
		&cgierr ("invalid directory name: $input. Reason: contains illegal characters.");
	}
	if (!($input =~ m/^\Q$build_root_path\E/)) {
		&cgierr ("invalid directory name: $input. Reason: does not start with defined root.");
	}
		
	return $input;
}

1;