<?

/*
Apache directory is a script trick.

in httpd.conf
<Location /local>
ForceType application/x-httpd-php
</Location>

in .htaccess
<Files local>
ForceType application/x-httpd-php
</Files>

 */

require_once("lib.utils.php");
require_once("lib.settings.php");

class ArbitroWebURLMangler
{
	var $dataURL = FALSE;
	var $dataQUERY = FALSE;

	function ArbitroWebURLMangler($inputURL, $xtraQuery = FALSE)
	{
		$this->dataURL = $inputURL;
		if($xtraQuery)
		{
			$this->dataQUERY = $xtraQuery;
		}
	}

	function getAdjustedUrl($url)
	{
		extract(parse_url($url));

		$base = $this->dataURL;

		while(TRUE)
		{
			if (!isset($base))
			{
				if (!isset($scheme))
				{
					// default to 'file' scheme.
					$scheme="file";
				}
				break;
			}

			extract(parse_url($base), EXTR_PREFIX_ALL, "B");

			if (!isset($scheme))
			{
				$scheme = $B_scheme;
			} elseif ($scheme != $B_scheme)
			{
				break;
			}

			if (isset($host) || isset($port))
			{
				break;
			}

			if (isset($B_host))
			{
				$host = $B_host;
			}

			if (isset($B_port))
			{
				$port = $B_port;
			}

			if (!isset($path))
			{
				$path=$B_path;
				if (!isset($query) && isset($B_query))
				{
					$query=$B_query;
				}
			} elseif (!preg_match("@^/@", $path))
			{
				$ppath = "";
				if (isset($B_path))
				{
					$ppath = $B_path;
					$ppath = preg_replace("@/[^/]*$@", "/", $ppath);
				} else
				{
					$ppath = "/";
				}

				$path = $ppath.$path;

				$oldpath = "";
				do
				{
					$oldpath = $path;
					$path = preg_replace('@/\./@','/',$path);
				} while($path != $oldpath);

				$path = preg_replace('@/\.$@', '/', $path);
				do
				{
					$oldpath = $path;
					$path = preg_replace('@/[^/]*/\.\./@','/',$path);
				} while($path != $oldpath);

				$path = preg_replace('@/[^/]/\.\.$@','/',$path);

				// Icky! Icky!
				// People that go up past the root path are bad(tm).
				$path = preg_replace('@/\.\./@','/',$path);
			}
		} // infinite while(TRUE)

		if (!isset($path))
		{
			$path = '/';
		}

		$url_array = compact('scheme','host','port','path','query','fragment');
		return util_glueURL($url_array);
	}
}

function arbitroweb_encode($url, $encmode=FALSE)
{
	global $mangledURL;

	$newURL = $mangledURL->getAdjustedURL($url);

	switch($encmode)
	{
		case ARBITROWEB_MODE_ENCODE:
			return ARBITROWEB_URL_MAIN."/".util_encodeURL($newURL);
			break;
		case ARBITROWEB_MODE_RAW:
			return ARBITROWEB_URL_RAW."/".util_encodeURL($newURL);
		case ARBITROWEB_MODE_UNTOUCHED:
		default:
			return $newURL;
	}
}

function urlsearchstr($prefix)
{
	return "/( ".$prefix."[ ]?=[ \"\']?([0-9a-zA-Z\:\_\-\%\&\=\.\/\~\?\+]*)[ \"\']?)/i";
}

function replace_href($m)
{
	return " href=\"".arbitroweb_encode($m[2], ARBITROWEB_MODE_ENCODE)."\" ";
}

function adjust_href($line)
{
	return preg_replace_callback(urlsearchstr("href"), replace_href, $line);
}

function replace_src($m)
{
	return " src=\"".arbitroweb_encode($m[2], ARBITROWEB_MODE_RAW)."\" ";
}

function adjust_src($line)
{
	return preg_replace_callback(urlsearchstr("src"), replace_src, $line);
}

function replace_background($m)
{
	return " background=\"".arbitroweb_encode($m[2], ARBITROWEB_MODE_RAW)."\" ";
}

function adjust_background($line)
{
	return preg_replace_callback(urlsearchstr("background"), replace_background, $line);
}

function replace_action($m)
{
	return " action=\"".arbitroweb_encode($m[2], ARBITROWEB_MODE_ENCODE)."\" ";
}

function adjust_action($line)
{
	return preg_replace_callback(urlsearchstr("action"), replace_action, $line);
}

function replace_meta_refresh($m)
{
	return " content=\"".$m[2].";".arbitroweb_encode($m[3], ARBITROWEB_MODE_ENCODE)."\" ";
}

function adjust_meta_refresh($line)
{
	return preg_replace_callback("/( content[ ]?=[ \"\']?([0-9]+);URL=([0-9a-zA-Z\:\_\-\%\&\=\.\/\~\?\+]*)[ \"\']?)/i",
		replace_meta_refresh, $line);
}

function adjust_all($line)
{
	$ret = $line;
	$ret = adjust_src($ret);
	$ret = adjust_href($ret);
	$ret = adjust_action($ret);
	$ret = adjust_background($ret);
	$ret = adjust_meta_refresh($ret);
	return $ret;
}

function ERR($str)
{
	if(ARBITROWEB_FLAG_LOGGING)
	{
		error_log("[arbitroweb]:".$str."\n", 3, ARBITROWEB_FILENAME_LOG);
	}
}

/* ==== ENTRY POINT ==== */

$URL = FALSE;

if(isset($PATH_INFO))
{
	$slen = strlen($PATH_INFO);
	$tmpURL = substr($PATH_INFO, 1, strlen($PATH_INFO)-1);
	if(ereg("/$", $PATH_INFO))
	{
		$tmpURL = substr($tmpURL, 0, strlen($tmpURL)-1);
	}
	$URL = util_decodeURL($tmpURL);
} else
{
	header("Location: ".ARBITROWEB_URL_INDEX);
	exit();
}

if(isset($QUERY_STRING))
{
	$requestedQUERY = $QUERY_STRING;
} else
{
	$requestedQUERY = FALSE;
}

$mangledURL = new ArbitroWebURLMangler($URL, $requestedQUERY);

if($URL)
{
	$realURL = $URL;
	if($requestedQUERY)
	{
		$realURL .= "?".$requestedQUERY;
	}

	$fp = fopen($realURL, "r");
	if($fp)
	{
		util_writelog("arbitroweb", $realURL, "200");
		while($line=fgets($fp, 4096))
		{
			$line = StripSlashes($line);

			echo adjust_all($line);
		}
		fclose($fp);
	}
	else
	{
		util_writelog("arbitroweb", $realURL, "500");
		$redirURL = ARBITROWEB_URL_INDEX."?failedURL=".util_encodeURL($URL);
		header("Location: $redirURL");
	}
}

?>